@@ -23,13 +23,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 24177 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Date: 2010-04-05 06:52:04 -0500 (Mon, 05 Apr 2010) $
  */
 
-#ifndef _PHTML
-#define _PHTML
+#ifndef PTLIB_HTML_H
+#define PTLIB_HTML_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -41,7 +41,7 @@
 // PHTML
 
 /** This class describes a HyperText markup Language string as used by the
-   World Wide Web and the #PURL# and #PHTTPSocket# class.
+   World Wide Web and the <code>PURL</code> and <code>PHTTP</code> class.
    
    All of the standard stream I/O operators, manipulators etc will operate on
    the PString class.
@@ -901,7 +901,32 @@ class PHTML : public PStringStream
     };
 
 
-    class InputRange : public InputField {
+    class InputNumber : public InputField {
+      public:
+        InputNumber(
+          const char * fname,
+          int min, int max,
+          int value = 0,
+          DisableCodes disabled = Enabled,
+          const char * attr = NULL
+        );
+        virtual ~InputNumber() {}
+      protected:
+        InputNumber(
+          const char * type,
+          const char * fname,
+          int min, int max,
+          int value,
+          DisableCodes disabled,
+          const char * attr
+        );
+        virtual void AddAttr(PHTML & html) const;
+      private:
+        void Construct(int min, int max, int value);
+        int minValue, maxValue, initValue;
+    };
+
+    class InputRange : public InputNumber {
       public:
         InputRange(
           const char * fname,
@@ -911,10 +936,6 @@ class PHTML : public PStringStream
           const char * attr = NULL
         );
         virtual ~InputRange() {}
-      protected:
-        virtual void AddAttr(PHTML & html) const;
-      private:
-        int minValue, maxValue, initValue;
     };
 
     class InputFile : public InputField {
@@ -1012,7 +1033,7 @@ class PHTML : public PStringStream
 };
 
 
-#endif
+#endif // PTLIB_HTML_H
 
 
 // End Of File ///////////////////////////////////////////////////////////////
