@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25231 $
+ * $Revision: 28179 $
  * $Author: rjongbloed $
- * $Date: 2011-03-02 08:41:46 -0600 (Wed, 02 Mar 2011) $
+ * $Date: 2012-08-10 06:23:27 -0500 (Fri, 10 Aug 2012) $
  */
 
 #ifndef PTLIB_IPSOCKET_H
@@ -289,11 +289,27 @@ class PIPSocket : public PSocket
     static void SetDefaultIpAddressFamilyV6(); // PF_INET6
     static PBoolean IsIpAddressFamilyV6Supported();
 
-	static void SetDefaultV6ScopeId(int scopeId); // local-link adresses require one
+    static void SetDefaultV6ScopeId(int scopeId); // local-link adresses require one
     static int GetDefaultV6ScopeId(); 
 #endif
     static PIPSocket::Address GetDefaultIpAny();
 
+    /**Set flag for suppress getting canonical name when doing lookup via
+       hostname.
+
+       Some badly configured DNS servers can cause long delays when this
+       feature is used.
+      */
+    static void SetSuppressCanonicalName(bool suppress);
+
+    /**Get flag for suppress getting canonical name when doing lookup via
+       hostname.
+
+       Some badly configured DNS servers can cause long delays when this
+       feature is used.
+      */
+    static bool GetSuppressCanonicalName();
+
     /**Open an IPv4 or IPv6 socket
      */
     virtual PBoolean OpenSocket(
@@ -642,23 +658,33 @@ class PIPSocket : public PSocket
           ostream &strm   // Stream to print the object into.
         ) const;
 
-        /// Get the name of the interface.
-        const PString & GetName() const { return name; }
+        /** Get the name of the interface.
+            Note the name will havebeen sanitised of certain possible
+            characters that can cause issues elsewhere in the system.
+            Therefore, make sure that if you get a device name from some
+            other source than the InterfaceEntry, the name is sanitised via
+            PIPSocket::InterfaceEntry::SanitiseName() before comparing against
+            the name returned here.
+          */
+        const PString & GetName() const { return m_name; }
 
         /// Get the address associated with the interface.
-        Address GetAddress() const { return ipAddr; }
+        Address GetAddress() const { return m_ipAddress; }
 
         /// Get the net mask associated with the interface.
-        Address GetNetMask() const { return netMask; }
+        Address GetNetMask() const { return m_netMask; }
 
         /// Get the MAC address associate with the interface.
-        const PString & GetMACAddress() const { return macAddr; }
+        const PString & GetMACAddress() const { return m_macAddress; }
+
+        /// Sanitise a device name for use in PTLib
+        static void SanitiseName(PString & name);
 
       protected:
-        PString name;
-        Address ipAddr;
-        Address netMask;
-        PString macAddr;
+        PString m_name;
+        Address m_ipAddress;
+        Address m_netMask;
+        PString m_macAddress;
 
       friend class PIPSocket;
     };
