@@ -26,13 +26,17 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 23334 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 05:40:38 -0500 (Wed, 04 Jun 2008) $
+ * $Date: 2009-09-04 08:36:47 -0500 (Fri, 04 Sep 2009) $
  */
 
 ///////////////////////////////////////////////////////////////////////////////
 // PTime
 
+  public:
+    PTime(const FILETIME & timestamp);
+    void SetFromFileTime(const FILETIME & timestamp);
+
 
 // End Of File ///////////////////////////////////////////////////////////////
