@@ -20,9 +20,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 25553 $
  * $Author: rjongbloed $
- * $Date: 2008-12-11 23:42:13 -0600 (Thu, 11 Dec 2008) $
+ * $Date: 2011-04-13 03:19:16 -0500 (Wed, 13 Apr 2011) $
  */
 
 #include <ptbuildopts.h>
@@ -93,8 +93,6 @@ class PVideoInputDevice_Application : public PVideoInputDevice
       */
     virtual PBoolean IsCapturing();
 
-    virtual PBoolean SetVideoFormat(VideoFormat newFormat);
-
     /**Set the colour format to be used.
        Note that this function does not do any conversion. If it returns TRUE
        then the video device does the colour format in native mode.
@@ -109,28 +107,6 @@ class PVideoInputDevice_Application : public PVideoInputDevice
       const PString & colourFormat ///< New colour format for device.
     );
 
-    /**Set the video frame rate to be used on the device.
-
-       Default behaviour sets the value of the frameRate variable and then
-       returns TRUE.
-    */
-    virtual PBoolean SetFrameRate(
-      unsigned rate  ///< Frames  per second
-    );
-
-    /**Set the frame size to be used.
-
-       Note that devices may not be able to produce the requested size, and
-       this function will fail.  See SetFrameSizeConverter().
-
-       Default behaviour sets the frameWidth and frameHeight variables and
-       returns TRUE.
-    */
-    virtual PBoolean SetFrameSize(
-      unsigned width,   ///< New width of frame
-      unsigned height   ///< New height of frame
-    );
-
     /**Get the maximum frame size in bytes.
 
        Note a particular device may be able to provide variable length
@@ -152,14 +128,6 @@ class PVideoInputDevice_Application : public PVideoInputDevice
       PINDEX * bytesReturned         ///< OPtional bytes returned.
     );
 
-    /**Try all known video formats & see which ones are accepted by the video driver
-     */
-    virtual PBoolean TestAllFormats();
-
-    /**Set the video channel (not used)
-      */
-    virtual PBoolean SetChannel(int newChannel);
-
     /**Set the Handle of the window you wish to capture
       */
     void AttachCaptureWindow(
@@ -168,17 +136,15 @@ class PVideoInputDevice_Application : public PVideoInputDevice
     );
 
   protected:
-      HWND m_hWnd;                   ///< Handle of Window to Capture
-      bool m_client;                 ///< Capture the client area only
-      PBYTEArray bitMapInfoStorage;  ///< used for storing raw DIB
-      PBYTEArray tempPixelBuffer;    ///< used for storing temporary version of bitmap during convertion
+    bool GetWindowBitmap(BITMAP & bitmapInfo, BYTE * pixels = NULL, bool useTemp = false);
 
-      PMutex lastFrameMutex;     ///< Frame Grab Mutex
-      PAdaptiveDelay grabDelay;  ///< Frame Grab delay
+    HWND       m_hWnd;               ///< Handle of Window to Capture
+    bool       m_client;             ///< Capture the client area only
+    PBYTEArray m_tempPixelBuffer;    ///< used for storing temporary version of bitmap during convertion
+
+    PMutex         m_lastFrameMutex; ///< Frame Grab Mutex
+    PAdaptiveDelay m_grabDelay;      ///< Frame Grab delay
 };
 
 
-PWLIB_STATIC_LOAD_PLUGIN(Application, PVideoInputDevice);
-
-
 #endif // P_APPSHARE
