@@ -23,9 +23,9 @@
  *                 Mark Cooke (mpc@star.sr.bham.ac.uk)
  *                 Nicola Orru' <nigu@itadinanta.it>
  *
- * $Revision: 20950 $
- * $Author: ms30002000 $
- * $Date: 2008-09-14 05:03:36 -0500 (Sun, 14 Sep 2008) $
+ * $Revision: 27739 $
+ * $Author: rjongbloed $
+ * $Date: 2012-05-30 18:49:09 -0500 (Wed, 30 May 2012) $
  */
 #ifndef _PVIDEOIOV4L2
 #define _PVIDEOIOV4L2
@@ -49,11 +49,12 @@ class PVideoInputDevice_V4L2: public PVideoInputDevice
 {
 
   PCLASSINFO(PVideoInputDevice_V4L2, PVideoInputDevice);
-      
-
+private:
+  PVideoInputDevice_V4L2(const PVideoInputDevice_V4L2& ){};
+  PVideoInputDevice_V4L2& operator=(const PVideoInputDevice_V4L2& ){ return *this; };
 public:
   PVideoInputDevice_V4L2();
-  ~PVideoInputDevice_V4L2();
+  virtual ~PVideoInputDevice_V4L2();
   
   void ReadDeviceDirectory (PDirectory, POrdinalToString &);
 
@@ -84,8 +85,8 @@ public:
   PBoolean TestAllFormats();
 
   PBoolean SetFrameSize(unsigned int, unsigned int);
+  PBoolean SetNearestFrameSize(unsigned int, unsigned int);
   PBoolean SetFrameRate(unsigned int);
-  PBoolean VerifyHardwareFrameSize(unsigned int, unsigned int);
 
   PBoolean GetParameters(int*, int*, int*, int*, int*);
 
@@ -112,10 +113,18 @@ public:
 
   PBoolean NormalReadProcess(BYTE*, PINDEX*);
 
+private:
   void ClearMapping();
 
   PBoolean SetMapping();
 
+  PBoolean VerifyHardwareFrameSize(unsigned int & width, unsigned int & height);
+
+  PBoolean QueueBuffers();
+
+  PBoolean StartStreaming();
+  void StopStreaming();
+
   struct v4l2_capability videoCapability;
   struct v4l2_streamparm videoStreamParm;
   PBoolean   canRead;
@@ -128,6 +137,11 @@ public:
   uint   videoBufferCount;
   uint   currentvideoBuffer;
 
+  PMutex mmapMutex;                             /** Has MMAP frame buffers in use? */
+  PBoolean isOpen;				/** Has the Video Input Device successfully been openend? */
+  PBoolean areBuffersQueued;
+  PBoolean isStreaming;
+
   int    videoFd;
   int    frameBytes;
   PBoolean   started;
