@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 26456 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 05:40:38 -0500 (Wed, 04 Jun 2008) $
+ * $Date: 2011-09-22 00:27:10 -0500 (Thu, 22 Sep 2011) $
  */
 
 #ifndef _Vidtest_MAIN_H
@@ -43,11 +43,13 @@ class VidTest : public PProcess
  protected:
    PDECLARE_NOTIFIER(PThread, VidTest, GrabAndDisplay);
 
-  PVideoInputDevice  * grabber;
-  PVideoOutputDevice * display;
-  PSyncPointAck        exitGrabAndDisplay;
-  PSyncPointAck        pauseGrabAndDisplay;
-  PSyncPoint           resumeGrabAndDisplay;
+  PVideoInputDevice     * m_grabber;
+  PVideoOutputDevice    * m_display;
+  PVideoOutputDevice    * m_secondary;
+  PList<PColourConverter> m_converters;
+  PSyncPointAck           m_exitGrabAndDisplay;
+  PSyncPointAck           m_pauseGrabAndDisplay;
+  PSyncPoint              m_resumeGrabAndDisplay;
 };
 
 
