@@ -5,9 +5,9 @@
  *
  * Portable Windows Library
  *
- * $Revision: 24473 $
- * $Author: rjongbloed $
- * $Date: 2010-06-07 19:39:21 -0500 (Mon, 07 Jun 2010) $
+ * $Revision: 25133 $
+ * $Author: shorne $
+ * $Date: 2011-02-20 01:51:05 -0600 (Sun, 20 Feb 2011) $
  */
 
 ///////////////////////////////////////////////////////////////////////
@@ -692,7 +692,7 @@ PBoolean PASN_Choice::DecodePER(PPER_Stream & strm)
 
       tag += numChoices;
 
-      unsigned len;
+      unsigned len = 0;
       if (!strm.LengthDecode(0, INT_MAX, len))
         return PFalse;
 
@@ -724,7 +724,7 @@ PBoolean PASN_Choice::DecodePER(PPER_Stream & strm)
       return PFalse;
   }
 
-  return CreateObject() && choice->Decode(strm);
+  return CreateObject() && choice && choice->Decode(strm);
 }
 
 
@@ -952,7 +952,7 @@ PBoolean PPER_Stream::ArrayDecode(PASN_Array & array)
 {
   array.RemoveAll();
 
-  unsigned size;
+  unsigned size = 0;
   if (!array.ConstrainedLengthDecode(*this, size))
     return PFalse;
 
@@ -1178,7 +1178,7 @@ PBoolean PPER_Stream::SmallUnsignedDecode(unsigned & value)
   if (!SingleBitDecode())
     return MultiBitDecode(6, value);      // 10.6.1
 
-  unsigned len;
+  unsigned len = 0;
   if (!LengthDecode(0, INT_MAX, len))  // 10.6.2
     return PFalse;
 
