@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23090 $
+ * $Revision: 24983 $
  * $Author: rjongbloed $
- * $Date: 2009-07-08 11:30:47 -0500 (Wed, 08 Jul 2009) $
+ * $Date: 2010-12-22 23:23:23 -0600 (Wed, 22 Dec 2010) $
  */
 
 #ifdef __GNUC__
@@ -51,11 +51,19 @@ PBase64::PBase64()
 }
 
 
-void PBase64::StartEncoding(PBoolean useCRLF)
+void PBase64::StartEncoding(bool useCRLF)
 {
   encodedString = "";
   encodeLength = nextLine = saveCount = 0;
-  useCRLFs = useCRLF;
+  endOfLine = useCRLF ? "\r\n" : "\n";
+}
+
+
+void PBase64::StartEncoding(const char * eol)
+{
+  encodedString = "";
+  encodeLength = nextLine = saveCount = 0;
+  endOfLine = eol;
 }
 
 
@@ -67,7 +75,7 @@ void PBase64::ProcessEncoding(const PString & str)
 
 void PBase64::ProcessEncoding(const char * cstr)
 {
-  ProcessEncoding((const BYTE *)cstr, strlen(cstr));
+  ProcessEncoding((const BYTE *)cstr, (int)strlen(cstr));
 }
 
 
@@ -89,10 +97,10 @@ void PBase64::OutputBase64(const BYTE * data)
   out[encodeLength++] = Binary2Base64[((data[1]&15)<<2) | (data[2]>>6)];
   out[encodeLength++] = Binary2Base64[data[2]&0x3f];
 
-  if (++nextLine > 18) { // 76 columns
-    if (useCRLFs)
-      out[encodeLength++] = '\r';
-    out[encodeLength++] = '\n';
+  PINDEX len = endOfLine.GetLength();
+  if (++nextLine > (76-len)/4) {
+    for (PINDEX i = 0; i < len; ++i)
+      out[encodeLength++] = endOfLine[i];
     nextLine = 0;
   }
 }
@@ -165,27 +173,28 @@ PString PBase64::CompleteEncoding()
 }
 
 
-PString PBase64::Encode(const PString & str)
+PString PBase64::Encode(const PString & str, const char * endOfLine)
 {
-  return Encode((const char *)str);
+  return Encode((const char *)str, str.GetLength(), endOfLine);
 }
 
 
-PString PBase64::Encode(const char * cstr)
+PString PBase64::Encode(const char * cstr, const char * endOfLine)
 {
-  return Encode((const BYTE *)cstr, strlen(cstr));
+  return Encode((const BYTE *)cstr, (PINDEX)strlen(cstr), endOfLine);
 }
 
 
-PString PBase64::Encode(const PBYTEArray & data)
+PString PBase64::Encode(const PBYTEArray & data, const char * endOfLine)
 {
-  return Encode(data, data.GetSize());
+  return Encode(data, data.GetSize(), endOfLine);
 }
 
 
-PString PBase64::Encode(const void * data, PINDEX length)
+PString PBase64::Encode(const void * data, PINDEX length, const char * endOfLine)
 {
   PBase64 encoder;
+  encoder.StartEncoding(endOfLine);
   encoder.ProcessEncoding(data, length);
   return encoder.CompleteEncoding();
 }
@@ -335,7 +344,7 @@ void PMessageDigest::Process(const PString & str)
 
 void PMessageDigest::Process(const char * cstr)
 {
-  Process(cstr, strlen(cstr));
+  Process(cstr, (int)strlen(cstr));
 }
 
 
@@ -597,13 +606,13 @@ void PMessageDigest5::Encode(const PString & str, Result & result)
 
 PString PMessageDigest5::Encode(const char * cstr)
 {
-  return Encode((const BYTE *)cstr, strlen(cstr));
+  return Encode((const BYTE *)cstr, (int)strlen(cstr));
 }
 
 
 void PMessageDigest5::Encode(const char * cstr, Result & result)
 {
-  Encode((const BYTE *)cstr, strlen(cstr), result);
+  Encode((const BYTE *)cstr, (int)strlen(cstr), result);
 }
 
 
@@ -643,7 +652,7 @@ void PMessageDigest5::Encode(const PString & str, Code & result)
 
 void PMessageDigest5::Encode(const char * cstr, Code & result)
 {
-  Encode((const BYTE *)cstr, strlen(cstr), result);
+  Encode((const BYTE *)cstr, (int)strlen(cstr), result);
 }
 
 void PMessageDigest5::Encode(const PBYTEArray & data, Code & result)
