@@ -11,9 +11,9 @@
  * Originally written by Poul-Henning Kamp <phk@freebsd.org>
  * Made into a C++ class by Roger Hardiman <roger@freebsd.org>, January 2002
  *
- * $Revision: 20385 $
+ * $Revision: 23725 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 05:40:38 -0500 (Wed, 04 Jun 2008) $
+ * $Date: 2009-10-28 22:11:42 -0500 (Wed, 28 Oct 2009) $
  */
 
 #ifdef __GNUC__
@@ -23,6 +23,8 @@
 #include <ptlib.h>
 #include <ptclib/dtmf.h>
 
+#if P_DTMF
+
 #include <math.h>
 
 /* Integer math scaling factor */
@@ -34,20 +36,17 @@
 #define P2 ((int)(POLRAD*POLRAD*FSC))
 
 PDTMFDecoder::PDTMFDecoder()
+  : sampleCount(0)
+  , tonesDetected(0)
+  , inputAmplitude(0)
 {
   // Initialise the class
-  int i,kk;
-  for (kk = 0; kk < NumTones; kk++) {
-    y[kk] = h[kk] = k[kk] = 0;
-  }
+  int i;
+  for (i = 0; i < NumTones; i++)
+    y[i] = h[i] = k[i] = 0;
 
-  nn = 0;
-  ia = 0;
-  so = 0;
-
-  for (i = 0; i < 256; i++) {
+  for (i = 0; i < 256; i++)
     key[i] = '?';
-  }
 
   /* We encode the tones in 8 bits, translate those to symbol */
   key[0x11] = '1'; key[0x12] = '4'; key[0x14] = '7'; key[0x18] = '*';
@@ -58,7 +57,7 @@ PDTMFDecoder::PDTMFDecoder()
   /* The frequencies we're trying to detect */
   /* These are precalculated to save processing power */
   /* static int dtmf[9] = {697, 770, 852, 941, 1209, 1336, 1477, 1633, 1100, 2100}; */
-  /* p1[kk] = (-cos(2 * 3.141592 * dtmf[kk] / 8000.0) * FSC) */
+  /* p1[tone] = (-cos(2 * 3.141592 * dtmf[tone] / 8000.0) * FSC) */
   p1[0] = -3497; p1[1] = -3369; p1[2] = -3212; p1[3] = -3027;
   p1[4] = -2384; p1[5] = -2040; p1[6] = -1635; p1[7] = -1164;
   p1[8] = -2660; p1[9] = 321;
@@ -66,10 +65,6 @@ PDTMFDecoder::PDTMFDecoder()
 
 PString PDTMFDecoder::Decode(const short * sampleData, PINDEX numSamples, unsigned mult, unsigned div)
 {
-  int x;
-  int s, kk;
-  int c, d, f, n;
-
 #if 0
   {
     static int fd = -1;
@@ -88,54 +83,55 @@ PString PDTMFDecoder::Decode(const short * sampleData, PINDEX numSamples, unsign
   for (pos = 0; pos < numSamples; pos++) {
 
     /* Read (and scale) the next 16 bit sample */
-    x = (int)(mult * (*sampleData++)) / div;
+    int x = (int)(mult * (*sampleData++)) / div;
     x = x / (32768/FSC);
 
     /* Input amplitude */
     if (x > 0)
-      ia += (x - ia) / 128;
+      inputAmplitude += (x - inputAmplitude) / 128;
     else
-      ia += (-x - ia) / 128;
+      inputAmplitude += (-x - inputAmplitude) / 128;
 
     /* For each tone */
-    s = 0;
-    for(kk = 0; kk < NumTones; kk++) {
+    int newTones = 0;
+    for (int tone = 0; tone < NumTones; tone++) {
 
       /* Turn the crank */
-      c = (P2 * (x - k[kk])) / FSC;
-      d = x + c;
-      f = (p1[kk] * (d - h[kk])) / FSC;
-      n = x - k[kk] - c;
-      k[kk] = h[kk] + f;
-      h[kk] = f + d;
+      int c = (P2 * (x - k[tone])) / FSC;
+      int d = x + c;
+      int f = (p1[tone] * (d - h[tone])) / FSC;
+      int n = x - k[tone] - c;
+      k[tone] = h[tone] + f;
+      h[tone] = f + d;
 
       /* Detect and Average */
       if (n > 0)
-        y[kk] += (n - y[kk]) / 64;
+        y[tone] += (n - y[tone]) / 64;
       else
-        y[kk] += (-n - y[kk]) / 64;
+        y[tone] += (-n - y[tone]) / 64;
 
       /* Threshold */
-      if (y[kk] > FSC/10 && y[kk] > ia) 
-        s |= 1 << kk;
+      if (y[tone] > FSC/10 && y[tone] > inputAmplitude) 
+        newTones |= 1 << tone;
     }
 
     /* Hysteresis and noise supressor */
-    if (s != so) {
-      nn = 0;
-      so = s;
-    } else if (nn++ == 520) {
-      if (s < 256) {
-        if (key[s] != '?') {
-          PTRACE(3,"DTMF\tDetected '" << key[s] << "' in PCM-16 stream");
-          keyString += key[s];
+    if (newTones != tonesDetected) {
+      sampleCount = 0;
+      tonesDetected = newTones;
+    }
+    else if (sampleCount++ == DetectSamples) {
+      if (tonesDetected < 256) {
+        if (key[tonesDetected] != '?') {
+          PTRACE(3,"DTMF\tDetected '" << key[tonesDetected] << "' in PCM-16 stream");
+          keyString += key[tonesDetected];
         }
       }
       else {
         char ch = 0;
-        if ((s & 0x100) != 0)
+        if ((tonesDetected & 0x100) != 0)
           ch = 'X';
-        else if ((s & 0x200) != 0)
+        else if ((tonesDetected & 0x200) != 0)
           ch = 'Y';
 
         if (ch != 0) {
@@ -150,9 +146,9 @@ PString PDTMFDecoder::Decode(const short * sampleData, PINDEX numSamples, unsign
 
 ////////////////////////////////////////////////////////////////////////////////////////////
 
-static int sine(int angle)
+static int sine(int angle, int freq)
 {
-  static int sinArray[PTones::MaxFrequency] = {
+  static int const sinArray[2000] = {
     0,0,1,2,3,3,4,5,6,7,7,8,9,10,10,11,12,13,14,14,15,16,17,18,18,19,20,21,21,22,
     23,24,25,25,26,27,28,29,29,30,31,32,32,33,34,35,36,36,37,38,39,40,40,41,42,43,43,44,45,46,
     47,47,48,49,50,51,51,52,53,54,54,55,56,57,58,58,59,60,61,62,62,63,64,65,65,66,67,68,69,69,
@@ -221,19 +217,21 @@ static int sine(int angle)
     999,999,999,999,999,999,999,999,999,999,999,999,999,999,999,999,999,999,999,999,999,999,999,999,999,999,999,999,999,999,
     999,999,999,999,999,999,999,999,999,999,999,999,999,999,999,999,999,999,999,999
   };
+  static int const sinArraySize = sizeof(sinArray)/sizeof(sinArray[0]);
 
-  int quadrant = angle / PTones::MaxFrequency;
-  int offset   = angle % PTones::MaxFrequency;
+  int adjustedAngle = (int)(angle*sinArraySize*4LL/freq);
+  int quadrant = adjustedAngle / sinArraySize;
+  int offset   = adjustedAngle % sinArraySize;
 
   switch (quadrant) {
     case 0:
       return sinArray[offset];
     case 1:
-      return sinArray[PTones::MaxFrequency-1-offset];
+      return sinArray[sinArraySize-1-offset];
     case 2: 
       return -sinArray[offset];
     default:
-      return -sinArray[PTones::MaxFrequency-1-offset];
+      return -sinArray[sinArraySize-1-offset];
   }
 }
 
@@ -241,27 +239,48 @@ static int sine(int angle)
 ////////////////////////////////////////////////////////////////////////
 
     
-PTones::PTones(unsigned volume): 
-  PShortArray(),
-  masterVolume(volume),
-  lastOperation(0),
-  lastFrequency1(0),
-  lastFrequency2(0),
-  angle1(0),
-  angle2(0)
+PTones::PTones(unsigned volume, unsigned sampleRate)
+  : m_sampleRate(sampleRate)
+  , m_masterVolume(volume)
 {
+  Construct();
 }
 
 
-PTones::PTones(const PString & descriptor, unsigned volume)
-  : masterVolume(volume)
+PTones::PTones(const PString & descriptor, unsigned volume, unsigned sampleRate)
+  : m_sampleRate(sampleRate)
+  , m_masterVolume(volume)
 {
+  Construct();
+
   if (!Generate(descriptor)) {
     PTRACE(1,"DTMF\tCannot encode tone \"" << descriptor << '"');
   }
 }
 
 
+void PTones::Construct()
+{
+  m_lastOperation = 0;
+  m_lastFrequency1 = 0;
+  m_lastFrequency2 = 0;
+  m_angle1 = 0;
+  m_angle2 = 0;
+
+  if (m_sampleRate < 8000)
+    m_sampleRate = 8000;
+  else if (m_sampleRate > 96000)
+    m_sampleRate = 96000;
+
+  m_maxFrequency = m_sampleRate/4;
+
+  if (m_masterVolume < 1)
+    m_masterVolume = 1;
+  else if (m_masterVolume > 100)
+    m_masterVolume = 100;
+}
+
+
 bool PTones::Generate(const PString & descriptor)
 {
   PStringArray toneChunks = descriptor.Tokenise('/');
@@ -335,15 +354,15 @@ bool PTones::Generate(const PString & descriptor)
 
 bool PTones::Generate(char operation, unsigned frequency1, unsigned frequency2, unsigned milliseconds, unsigned volume)
 {
-  if (lastOperation  != operation  ||
-      lastFrequency1 != frequency1 ||
-      lastFrequency2 != frequency2) {
-    lastOperation  = operation;
-    lastFrequency1 = frequency1;
-    lastFrequency2 = frequency2;
+  if (m_lastOperation  != operation  ||
+      m_lastFrequency1 != frequency1 ||
+      m_lastFrequency2 != frequency2) {
+    m_lastOperation  = operation;
+    m_lastFrequency1 = frequency1;
+    m_lastFrequency2 = frequency2;
 
-    angle1 = 0;
-    angle2 = 0;
+    m_angle1 = 0;
+    m_angle2 = 0;
   }
 
   switch (operation) {
@@ -366,26 +385,26 @@ bool PTones::Generate(char operation, unsigned frequency1, unsigned frequency2,
 
 bool PTones::Juxtapose(unsigned frequency1, unsigned frequency2, unsigned milliseconds, unsigned volume)
 {
-  if (frequency1 < MinFrequency || frequency1 > MaxFrequency ||
-      frequency2 < MinFrequency || frequency2 > MaxFrequency)
+  if (frequency1 < MinFrequency || frequency1 > m_maxFrequency ||
+      frequency2 < MinFrequency || frequency2 > m_maxFrequency)
     return false;
 
   // TODO this gived 8000 samples for 100 ms !!!
   //unsigned samples = CalcSamples(milliseconds, frequency1, frequency2);
-  unsigned samples = milliseconds * SampleRate / 1000;
+  unsigned samples = milliseconds * m_sampleRate / 1000;
   while (samples-- > 0) {
-    int a1 = sine(angle1);
-    int a2 = sine(angle2);
+    int a1 = sine(m_angle1, m_sampleRate);
+    int a2 = sine(m_angle2, m_sampleRate);
 
     AddSample((a1 + a2) / 2, volume);
 
-    angle1 += frequency1;
-    if (angle1 >= SampleRate) 
-      angle1 -= SampleRate;
+    m_angle1 += frequency1;
+    if (m_angle1 >= (int)m_sampleRate) 
+      m_angle1 -= m_sampleRate;
 
-    angle2 += frequency2;
-    if (angle2 >= SampleRate) 
-      angle2 -= SampleRate;
+    m_angle2 += frequency2;
+    if (m_angle2 >= (int)m_sampleRate) 
+      m_angle2 -= m_sampleRate;
   }
   return true;
 }
@@ -393,24 +412,24 @@ bool PTones::Juxtapose(unsigned frequency1, unsigned frequency2, unsigned millis
 
 bool PTones::Modulate(unsigned frequency1, unsigned modulator, unsigned milliseconds, unsigned volume)
 {
-  if (frequency1 < MinFrequency || frequency1 > MaxFrequency || modulator < MinModulation || modulator >= frequency1/2)
+  if (frequency1 > m_maxFrequency || frequency1 > m_maxFrequency || modulator < MinModulation || modulator >= frequency1/2)
     return false;
 
   unsigned samples = CalcSamples(milliseconds, frequency1, modulator);
 
   while (samples-- > 0) {
-    int a1 = sine(angle1);   // -999 to 999
-    int a2 = sine(angle2);   // -999 to 999
+    int a1 = sine(m_angle1, m_sampleRate);   // -999 to 999
+    int a2 = sine(m_angle2, m_sampleRate);   // -999 to 999
 
     AddSample((a1 * (a2 + SineScale)) / SineScale / 2, volume);
 
-    angle1 += frequency1;
-    if (angle1 >= SampleRate) 
-      angle1 -= SampleRate;
+    m_angle1 += frequency1;
+    if (m_angle1 >= (int)m_sampleRate) 
+      m_angle1 -= m_sampleRate;
 
-    angle2 += modulator;
-    if (angle2 >= SampleRate) 
-      angle2 -= SampleRate;
+    m_angle2 += modulator;
+    if (m_angle2 >= (int)m_sampleRate) 
+      m_angle2 -= m_sampleRate;
   }
   return true;
 }
@@ -458,16 +477,16 @@ bool PTones::PureTone(unsigned frequency1, unsigned milliseconds, unsigned volum
     return true;
   }
 
-  if (frequency1 < MinFrequency || frequency1 > MaxFrequency)
+  if (frequency1 < MinFrequency || frequency1 > m_maxFrequency)
     return false;
 
   unsigned samples = CalcSamples(milliseconds, frequency1);
   while (samples-- > 0) {
-    AddSample(sine(angle1), volume);
+    AddSample(sine(m_angle1, m_sampleRate), volume);
 
-    angle1 += frequency1;
-    if (angle1 >= SampleRate) 
-      angle1 -= SampleRate;
+    m_angle1 += frequency1;
+    if (m_angle1 >= (int)m_sampleRate) 
+      m_angle1 -= m_sampleRate;
   }
   return true;
 }
@@ -475,7 +494,7 @@ bool PTones::PureTone(unsigned frequency1, unsigned milliseconds, unsigned volum
 
 bool PTones::Silence(unsigned milliseconds)
 {
-  unsigned samples = milliseconds * SampleRate/1000;
+  unsigned samples = milliseconds * m_sampleRate/1000;
   while (samples-- > 0) 
     AddSample(0, 0);
   return true;
@@ -504,8 +523,8 @@ unsigned PTones::CalcSamples(unsigned ms, unsigned f1, unsigned f2)
   // now find the number of times we need to repeat this to match the sampling rate
   unsigned n1 = 1;
   unsigned n2 = 1;
-  while (n1*SampleRate*v1 != n2*f1) {
-    if (n1*SampleRate*v1 < n2*f1) 
+  while (n1*m_sampleRate*v1 != n2*f1) {
+    if (n1*m_sampleRate*v1 < n2*f1) 
       n1++;
     else
       n2++;
@@ -514,10 +533,10 @@ unsigned PTones::CalcSamples(unsigned ms, unsigned f1, unsigned f2)
   // v1 repetitions of t == v2 repetitions sample frequency
   //cout << n1*v1 << " cycles at " << f1 << "hz = "
   //     << n1*v2 << " cycles at " << f2 << "hz = "
-  //     << n2    << " samples at " << SampleRate << "hz" << endl;
+  //     << n2    << " samples at " << m_sampleRate << "hz" << endl;
 
   // Make sure we round up the number of milliseconds to even multiple of cycles
-  return ms == 0 ? n2 : ((ms * SampleRate/1000 + n2 - 1)/n2*n2);
+  return ms == 0 ? n2 : ((ms * m_sampleRate/1000 + n2 - 1)/n2*n2);
 }
 
 
@@ -527,7 +546,7 @@ void PTones::AddSample(int sample, unsigned volume)
   PINDEX length = GetSize();
   SetSize(length + 1);
   sample *= volume;
-  sample *= masterVolume;
+  sample *= m_masterVolume;
   sample /= SineScale*100*100/SHRT_MAX;
   SetAt(length, (short)sample);
 }
@@ -604,7 +623,7 @@ void PDTMFEncoder::AddTone(char digit, unsigned milliseconds)
 
 void PDTMFEncoder::AddTone(double f1, double f2, unsigned milliseconds)
 {
-  if (f1 > 0 && f1 < MaxFrequency && f2 > 0 && f2 < MaxFrequency){
+  if (f1 > 0 && f1 < m_maxFrequency && f2 > 0 && f2 < m_maxFrequency){
     Generate('+', (unsigned)f1, (unsigned)f2, milliseconds);
   } else {
     PAssertAlways(PInvalidParameter);
@@ -621,4 +640,6 @@ char PDTMFEncoder::DtmfChar(PINDEX i)
 }
 
 
+#endif // P_DTMF
+
 ////////////////////////////////////////////////////////////////////////////
