@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 24534 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 05:40:38 -0500 (Wed, 04 Jun 2008) $
+ * $Date: 2010-06-23 06:14:27 -0500 (Wed, 23 Jun 2010) $
  */
 
 #ifdef __GNUC__
@@ -105,6 +105,33 @@ struct NAPTR_DNS {
   PString GetReplacement() const    { return PString(GetReplacementBase()+1, GetReplacementLen()); }
 };
 
+
+void ResolveNAPTR(PDNS_RECORD dnsRecord, PDNS::NAPTRRecord & record)
+{
+#ifdef _WIN32
+  if (PProcess::IsOSVersion(6)) {
+    DNS_NAPTR_DATA * naptr = (DNS_NAPTR_DATA *)&dnsRecord->Data;
+    record.order       = naptr->wOrder; 
+    record.preference  = naptr->wPreference; 
+    record.flags       = naptr->pFlags; 
+    record.service     = naptr->pService; 
+    record.regex       = naptr->pRegularExpression; 
+    record.replacement = naptr->pReplacement; 
+  }
+  else
+#endif
+  {
+    NAPTR_DNS * naptr = (NAPTR_DNS *)&dnsRecord->Data; 
+    record.order       = naptr->order; 
+    record.preference  = naptr->preference; 
+    record.flags       = naptr->GetFlags(); 
+    record.service     = naptr->GetService(); 
+    record.regex       = naptr->GetRegex(); 
+    record.replacement = naptr->GetReplacement(); 
+  }
+}
+
+
 PDNS::NAPTRRecord * PDNS::NAPTRRecordList::HandleDNSRecord(PDNS_RECORD dnsRecord, PDNS_RECORD /*results*/)
 {
   PDNS::NAPTRRecord * record = NULL;
@@ -115,14 +142,8 @@ PDNS::NAPTRRecord * PDNS::NAPTRRecordList::HandleDNSRecord(PDNS_RECORD dnsRecord
       ) {
     record = new NAPTRRecord();
 
-    NAPTR_DNS * naptr = (NAPTR_DNS *)&dnsRecord->Data;
+	ResolveNAPTR(dnsRecord, *record);
 
-    record->order       = naptr->order;
-    record->preference  = naptr->preference;
-    record->flags       = naptr->GetFlags();
-    record->service     = naptr->GetService();
-    record->regex       = naptr->GetRegex();
-    record->replacement = naptr->GetReplacement();
   }
 
   return record;
@@ -256,7 +277,7 @@ static PString ApplyRegex(const PString & orig, const PString & regexStr)
 
 static PStringArray & GetENUMServers()
 {
-  static const char * defaultDomains[] = { "e164.voxgratia.net","e164.org","e164.arpa"};
+  static const char * defaultDomains[] = { "e164.org","e164.arpa"};
   static PStringArray servers(
           sizeof(defaultDomains)/sizeof(defaultDomains[0]),
           defaultDomains
