@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 23951 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2010-01-20 00:51:49 -0600 (Wed, 20 Jan 2010) $
  */
 
 #ifdef __GNUC__
@@ -972,11 +972,27 @@ PHTML::CheckBox::CheckBox(const char * fname,
 }
 
 
-PHTML::InputRange::InputRange(const char * fname,
-                              int min, int max, int value,
-                              DisableCodes disabled,
-                              const char * attr)
-  : InputField("range", fname, disabled, attr)
+PHTML::InputNumber::InputNumber(const char * fname,
+                                int min, int max, int value,
+                                DisableCodes disabled,
+                                const char * attr)
+  : InputField("number", fname, disabled, attr)
+{
+  Construct(min, max, value);
+}
+
+PHTML::InputNumber::InputNumber(const char * type,
+                                const char * fname,
+                                int min, int max,
+                                int value,
+                                DisableCodes disabled,
+                                const char * attr)
+  : InputField(type, fname, disabled, attr)
+{
+  Construct(min, max, value);
+}
+
+void PHTML::InputNumber::Construct(int min, int max, int value)
 {
   PAssert(min <= max, PInvalidParameter);
   minValue = min;
@@ -989,7 +1005,7 @@ PHTML::InputRange::InputRange(const char * fname,
     initValue = value;
 }
 
-void PHTML::InputRange::AddAttr(PHTML & html) const
+void PHTML::InputNumber::AddAttr(PHTML & html) const
 {
   InputField::AddAttr(html);
   PINDEX max = PMAX(-minValue, maxValue);
@@ -1005,6 +1021,15 @@ void PHTML::InputRange::AddAttr(PHTML & html) const
 }
 
 
+PHTML::InputRange::InputRange(const char * fname,
+                              int min, int max, int value,
+                              DisableCodes disabled,
+                              const char * attr)
+  : InputNumber("range", fname, min, max, value, disabled, attr)
+{
+}
+
+
 PHTML::InputFile::InputFile(const char * fname,
                             const char * accept,
                             DisableCodes disabled,
