@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20503 $
- * $Author: shorne $
- * $Date: 2008-06-26 07:52:56 -0500 (Thu, 26 Jun 2008) $
+ * $Revision: 27535 $
+ * $Author: rjongbloed $
+ * $Date: 2012-04-26 02:48:22 -0500 (Thu, 26 Apr 2012) $
  */
 
 #ifdef __GNUC__
@@ -47,7 +47,8 @@
 
 
 #if defined(_MSC_VER)
-#pragma comment(lib, P_LDAP_LIBRARY)
+  #pragma comment(lib, P_LDAP_LIBRARY)
+  #pragma message("LDAP support enabled")
 #endif
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -118,6 +119,15 @@ PBoolean PLDAPSession::SetOption(int optcode, void * value)
   return ldap_set_option(ldapContext, optcode, value);
 }
 
+PBoolean PLDAPSession::StartTLS()
+{
+#ifdef LDAP_EXOP_START_TLS
+  errorNumber = ldap_start_tls_s(ldapContext, NULL, NULL);
+  return errorNumber == LDAP_SUCCESS;
+#else
+  return LDAP_OPERATIONS_ERROR;
+#endif
+}
 
 PBoolean PLDAPSession::Bind(const PString & who,
                         const PString & passwd,
@@ -740,8 +750,13 @@ PBoolean PLDAPSession::GetNextSearchResult(SearchContext & context)
             errorNumber = LDAP_NO_RESULTS_RETURNED;
           context.completed = PTrue;
           return PFalse;
+        case LDAP_RES_SEARCH_REFERENCE :
+          errorNumber = LDAP_SUCCESS;
+          return PTrue;
+        // Ignore other result message types for now ...
+        default:
+          PTRACE(3, "Unhandled LDAP message type " << ldap_msgtype(context.message));
       }
-      // Ignore other result message types for now ...
     }
 
     ldap_msgfree(context.result);
@@ -1043,6 +1058,12 @@ void PLDAPSchema::OnSendSchema(PArray<PLDAPSession::ModAttrib> & attrib, PLDAPSe
 }
 
 
+#else
+
+  #if defined(_MSC_VER)
+    #pragma message("LDAP support DISABLED")
+  #endif
+
 #endif // P_LDAP
 
 
