@@ -27,9 +27,9 @@
  * Original copyright notice appears below
  *
  * $Id: pssl.cxx 19752 2008-03-15 15:30:24Z hfriederich $
- * $Revision: 23261 $
+ * $Revision: 26520 $
  * $Author: rjongbloed $
- * $Date: 2009-08-24 19:09:09 -0500 (Mon, 24 Aug 2009) $
+ * $Date: 2011-10-04 23:54:38 -0500 (Tue, 04 Oct 2011) $
  */
 
 /* crypto/buffer/bss_sock.c */
@@ -104,10 +104,9 @@ extern "C" {
 
 
 #ifdef _MSC_VER
-
-#pragma comment(lib, P_SSL_LIB1)
-#pragma comment(lib, P_SSL_LIB2)
-
+  #pragma comment(lib, P_SSL_LIB1)
+  #pragma comment(lib, P_SSL_LIB2)
+  #pragma message("SSL support (via OpenSSL) enabled")
 #endif
 
 
@@ -127,12 +126,13 @@ class PSSLInitialiser : public PProcessStartup
     virtual void OnShutdown();
     void LockingCallback(int mode, int n);
 
+    PFACTORY_GET_SINGLETON(PProcessStartupFactory, PSSLInitialiser);
+
   private:
     vector<PMutex> mutexes;
 };
 
-static const char SSLSubsystemName[] = "OpenSSL";
-static PFactory<PProcessStartup>::Worker<PSSLInitialiser> PSSLInitialiserInstance(SSLSubsystemName, true);
+PFACTORY_CREATE_SINGLETON(PProcessStartupFactory, PSSLInitialiser);
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -719,8 +719,7 @@ PBoolean PSSLDiffieHellman::Load(const PFilePath & dhFile,
 
 static void LockingCallback(int mode, int n, const char * /*file*/, int /*line*/)
 {
-  static PSSLInitialiser * instance = PFactory<PProcessStartup>::CreateInstanceAs<PSSLInitialiser>(SSLSubsystemName);
-  instance->LockingCallback(mode, n);
+  PSSLInitialiser::GetInstance().LockingCallback(mode, n);
 }
 
 
@@ -806,9 +805,6 @@ void PSSLContext::Construct(Method method, const void * sessionId, PINDEX idSize
   SSL_METHOD * meth;
 
   switch (method) {
-    case SSLv2:
-      meth = SSLv2_method();
-      break;
     case SSLv3:
       meth = SSLv3_method();
       break;
@@ -1284,59 +1280,11 @@ PBoolean PSSLChannel::OnOpen()
 }
 
 
-//////////////////////////////////////////////////////////////////////////
-//
-//  misc unused code
-//
+#else
 
-#if 0
-
-extern "C" {
-
-static verify_depth = 0;
-static verify_error = VERIFY_OK;
-
-// should be X509 * but we can just have them as char *. 
-int verify_callback(int ok, X509 * xs, X509 * xi, int depth, int error)
-{
-  char *s;
-
-  s = (char *)X509_NAME_oneline(X509_get_subject_name(xs));
-  if (s == NULL) {
-//    ERR_print_errors(bio_err);
-    return(0);
-  }
-  PError << "depth= " << depth << " " << (char *)s << endl;
-  free(s);
-  if (error == VERIFY_ERR_UNABLE_TO_GET_ISSUER) {
-    s=(char *)X509_NAME_oneline(X509_get_issuer_name(xs));
-    if (s == NULL) {
-      PError << "verify error" << endl;
-      //ERR_print_errors(bio_err);
-      return(0);
-    }
-    PError << "issuer = " << s << endl;
-    free(s);
-  }
-
-  if (!ok) {
-    PError << "verify error:num=" << error << " " <<
-    X509_cert_verify_error_string(error) << endl;
-    if (verify_depth <= depth) {
-      ok=1;
-      verify_error=VERIFY_OK;
-    } else {
-      ok=0;
-      verify_error=error;
-    }
-  }
-  PError << "verify return:" << ok << endl;
-  return(ok);
-}
-
-};
-
-#endif
+  #ifdef _MSC_VER
+    #pragma message("SSL support (via OpenSSL) DISABLED")
+  #endif
 
 #endif // P_SSL
 
