@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22392 $
+ * $Revision: 26282 $
  * $Author: rjongbloed $
- * $Date: 2009-04-06 23:43:05 -0500 (Mon, 06 Apr 2009) $
+ * $Date: 2011-08-08 02:09:32 -0500 (Mon, 08 Aug 2011) $
  */
 
 #include <ptlib.h>
@@ -54,18 +54,28 @@ PTimeInterval::PTimeInterval(const PString & str)
 }
 
 
+PInt64 PTimeInterval::GetMilliSeconds() const
+{ 
+  return m_milliseconds; 
+}
+
+void PTimeInterval::SetMilliSeconds(PInt64 msecs)
+{ 
+  m_milliseconds = msecs;
+}
+
 PObject::Comparison PTimeInterval::Compare(const PObject & obj) const
 {
   PAssert(PIsDescendant(&obj, PTimeInterval), PInvalidCast);
   const PTimeInterval & other = (const PTimeInterval &)obj;
-  return milliseconds < other.milliseconds ? LessThan :
-         milliseconds > other.milliseconds ? GreaterThan : EqualTo;
-}
+
+  return GetMilliSeconds() < other.GetMilliSeconds() ? LessThan :
+         GetMilliSeconds() > other.GetMilliSeconds() ? GreaterThan : EqualTo;}
 
 
 void PTimeInterval::PrintOn(ostream & stream) const
 {
-  int precision = stream.precision();
+  int precision = (int)stream.precision();
 
   Formats fmt = NormalFormat;
   if ((stream.flags()&ios::scientific) != 0)
@@ -75,7 +85,7 @@ void PTimeInterval::PrintOn(ostream & stream) const
     precision = -precision;
   }
 
-  stream << AsString(precision, fmt, stream.width());
+  stream << AsString(precision, fmt, (int)stream.width());
 }
 
 
@@ -109,7 +119,7 @@ PString PTimeInterval::AsString(int precision, Formats format, int width) const
   else if (precision < 0)
     precision = 0;
 
-  PInt64 ms = milliseconds;
+  PInt64 ms = GetMilliSeconds();
   if (ms < 0) {
     str << '-';
     ms = -ms;
@@ -187,12 +197,28 @@ PString PTimeInterval::AsString(int precision, Formats format, int width) const
 }
 
 
+DWORD PTimeInterval::GetInterval() const
+{
+  PInt64 msecs = GetMilliSeconds();
+
+  if (msecs <= 0)
+    return 0;
+
+  if (msecs >= UINT_MAX)
+    return UINT_MAX;
+
+  return (DWORD)msecs;
+}
+
+
 void PTimeInterval::SetInterval(PInt64 millisecs,
                                 long seconds,
                                 long minutes,
                                 long hours,
                                 int days)
 {
+  PInt64 milliseconds;
+
   milliseconds = days;
   milliseconds *= 24;
   milliseconds += hours;
@@ -202,6 +228,8 @@ void PTimeInterval::SetInterval(PInt64 millisecs,
   milliseconds += seconds;
   milliseconds *= 1000;
   milliseconds += millisecs;
+
+  SetMilliSeconds(milliseconds);
 }
 
 
@@ -275,6 +303,11 @@ PObject::Comparison PTime::Compare(const PObject & obj) const
 }
 
 
+void PTime::SetTimestamp(time_t seconds, long usecs)
+{ 
+  microseconds = usecs%1000000;
+  theTime = seconds + usecs/1000000;
+}
 
 
 PString PTime::AsString(TimeFormat format, int zone) const
@@ -285,10 +318,14 @@ PString PTime::AsString(TimeFormat format, int zone) const
   switch (format) {
     case RFC1123 :
       return AsString("wwwe, dd MMME yyyy hh:mm:ss z", zone);
+    case RFC3339 :
+      return AsString("yyyy-MM-ddThh:mm:ssZZ", zone);
     case ShortISO8601 :
-      return AsString("yyyyMMddThhmmssZ");
+      return AsString("yyyyMMddThhmmssZ", zone);
     case LongISO8601 :
-      return AsString("yyyy-MM-dd T hh:mm:ss Z");
+      return AsString("yyyy-MM-dd T hh:mm:ss Z", zone);
+    case EpochTime :
+      return psprintf("%u.%06lu", theTime, microseconds);
     default :
       break;
   }
@@ -396,6 +433,9 @@ PString PTime::AsString(TimeFormat format, int zone) const
 
 PString PTime::AsString(const char * format, int zone) const
 {
+  if (!IsValid())
+    return PString::Empty();
+
   PAssert(format != NULL, PInvalidParameter);
 
   PBoolean is12hour = strchr(format, 'a') != NULL;
@@ -411,14 +451,14 @@ PString PTime::AsString(const char * format, int zone) const
   struct tm ts;
   struct tm * t = os_gmtime(&realTime, &ts);
 
-  PINDEX repeatCount;
-
   while (*format != '\0') {
-    repeatCount = 1;
-    switch (*format) {
+    char formatLetter = *format;
+    PINDEX repeatCount = 1;
+    while (*++format == formatLetter)
+      repeatCount++;
+
+    switch (formatLetter) {
       case 'a' :
-        while (*++format == 'a')
-          ;
         if (t->tm_hour < 12)
           str << GetTimeAM();
         else
@@ -426,26 +466,18 @@ PString PTime::AsString(const char * format, int zone) const
         break;
 
       case 'h' :
-        while (*++format == 'h')
-          repeatCount++;
         str << setw(repeatCount) << (is12hour ? (t->tm_hour+11)%12+1 : t->tm_hour);
         break;
 
       case 'm' :
-        while (*++format == 'm')
-          repeatCount++;
         str << setw(repeatCount) << t->tm_min;
         break;
 
       case 's' :
-        while (*++format == 's')
-          repeatCount++;
         str << setw(repeatCount) << t->tm_sec;
         break;
 
       case 'w' :
-        while (*++format == 'w')
-          repeatCount++;
         if (repeatCount != 3 || *format != 'e')
           str << GetDayName((Weekdays)t->tm_wday, repeatCount <= 3 ? Abbreviated : FullName);
         else {
@@ -458,8 +490,6 @@ PString PTime::AsString(const char * format, int zone) const
         break;
 
       case 'M' :
-        while (*++format == 'M')
-          repeatCount++;
         if (repeatCount < 3)
           str << setw(repeatCount) << (t->tm_mon+1);
         else if (repeatCount > 3 || *format != 'E')
@@ -476,14 +506,10 @@ PString PTime::AsString(const char * format, int zone) const
         break;
 
       case 'd' :
-        while (*++format == 'd')
-          repeatCount++;
         str << setw(repeatCount) << t->tm_mday;
         break;
 
       case 'y' :
-        while (*++format == 'y')
-          repeatCount++;
         if (repeatCount < 3)
           str << setw(2) << (t->tm_year%100);
         else
@@ -492,8 +518,8 @@ PString PTime::AsString(const char * format, int zone) const
 
       case 'z' :
       case 'Z' :
-        if (zone == 0) {
-          if (*format == 'Z')
+        if (repeatCount == 1 && zone == 0) {
+          if (formatLetter == 'Z')
             str << 'Z';
           else
             str << "GMT";
@@ -501,15 +527,14 @@ PString PTime::AsString(const char * format, int zone) const
         else {
           str << (zone < 0 ? '-' : '+');
           zone = PABS(zone);
-          str << setw(2) << (zone/60) << setw(2) << (zone%60);
+          str << setw(2) << (zone/60);
+          if (repeatCount > 0)
+            str << ':';
+          str << setw(2) << (zone%60);
         }
-        while (toupper(*++format) == 'z')
-          ;
         break;
 
       case 'u' :
-        while (*++format == 'u')
-          repeatCount++;
         switch (repeatCount) {
           case 1 :
             str << (microseconds/100000);
@@ -527,11 +552,10 @@ PString PTime::AsString(const char * format, int zone) const
         break;
 
       case '\\' :
-        format++;
         // Escaped character, put straight through to output string
 
       default :
-        str << *format++;
+        str << formatLetter;
     }
   }
 
@@ -591,7 +615,6 @@ int STDAPICALLTYPE PTimeIsDayName(const char * str, int day, int abbrev)
 };
 
 
-
 void PTime::ReadFrom(istream & strm)
 {
   time_t now;
@@ -599,10 +622,17 @@ void PTime::ReadFrom(istream & strm)
   time(&now);
   microseconds = 0;
   strm >> ws;
-  theTime = PTimeParse(&strm, os_localtime(&now, &timeBuf), GetTimeZone());
+  theTime = PTimeParse(&strm, os_localtime(&now, &timeBuf), GetTimeZone(StandardTime));
 }
 
 
+bool PTime::Parse(const PString & str)
+{
+  PStringStream strm(str);
+  ReadFrom(strm);
+  return IsValid();
+}
+
 
 PTime PTime::operator+(const PTimeInterval & t) const
 {
