@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23518 $
+ * $Revision: 27948 $
  * $Author: rjongbloed $
- * $Date: 2009-09-23 01:52:27 -0500 (Wed, 23 Sep 2009) $
+ * $Date: 2012-06-30 22:54:23 -0500 (Sat, 30 Jun 2012) $
  */
 
 #ifdef __GNUC__
@@ -65,7 +65,7 @@ PBoolean PSafeObject::SafeReference()
 #endif
   }
 
-  PTRACE(6, "SafeColl\tIncrement reference count to " << tracedReferenceCount << " for " << GetClass() << ' ' << (void *)this);
+  PTRACE(7, "SafeColl\tIncrement reference count to " << tracedReferenceCount << " for " << GetClass() << ' ' << (void *)this);
   return PTrue;
 }
 
@@ -87,7 +87,7 @@ PBoolean PSafeObject::SafeDereference()
 #endif
   safetyMutex.Signal();
 
-  PTRACE(6, "SafeColl\tDecrement reference count to " << tracedReferenceCount << " for " << GetClass() << ' ' << (void *)this);
+  PTRACE(7, "SafeColl\tDecrement reference count to " << tracedReferenceCount << " for " << GetClass() << ' ' << (void *)this);
 
   return mayBeDeleted;
 }
@@ -95,7 +95,7 @@ PBoolean PSafeObject::SafeDereference()
 
 PBoolean PSafeObject::LockReadOnly() const
 {
-  PTRACE(6, "SafeColl\tWaiting read ("<<(void *)this<<")");
+  PTRACE(7, "SafeColl\tWaiting read ("<<(void *)this<<")");
   safetyMutex.Wait();
 
   if (safelyBeingRemoved) {
@@ -120,7 +120,7 @@ void PSafeObject::UnlockReadOnly() const
 
 PBoolean PSafeObject::LockReadWrite()
 {
-  PTRACE(6, "SafeColl\tWaiting readWrite ("<<(void *)this<<")");
+  PTRACE(7, "SafeColl\tWaiting readWrite ("<<(void *)this<<")");
   safetyMutex.Wait();
 
   if (safelyBeingRemoved) {
@@ -386,6 +386,30 @@ PINDEX PSafeCollection::GetSize() const
 }
 
 
+void PSafeCollection::CopySafeCollection(PCollection * other)
+{
+  DisallowDeleteObjects();
+
+  for (PINDEX i = 0; i < other->GetSize(); ++i) {
+    PSafeObject * obj = dynamic_cast<PSafeObject *>(other->GetAt(i));
+    if (obj != NULL && obj->SafeReference())
+      collection->Append(obj);
+  }
+}
+
+
+void PSafeCollection::CopySafeDictionary(PAbstractDictionary * other)
+{
+  DisallowDeleteObjects();
+
+  for (PINDEX i = 0; i < other->GetSize(); ++i) {
+    PSafeObject * obj = dynamic_cast<PSafeObject *>(&other->AbstractGetDataAt(i));
+    if (obj != NULL && obj->SafeReference())
+      collection->Insert(other->AbstractGetKeyAt(i), obj);
+  }
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 
 PSafePtrBase::PSafePtrBase(PSafeObject * obj, PSafetyMode mode)
@@ -452,6 +476,12 @@ PObject::Comparison PSafePtrBase::Compare(const PObject & obj) const
 }
 
 
+void PSafePtrBase::PrintOn(ostream &strm) const
+{
+  strm << currentObject;
+}
+
+
 void PSafePtrBase::Assign(const PSafePtrBase & enumerator)
 {
   if (this == &enumerator)
@@ -678,23 +708,33 @@ void PSafePtrBase::ExitSafetyMode(ExitSafetyModeOption ref)
   if (ref == WithDereference && currentObject->SafeDereference()) {
     PSafeObject * objectToDelete = currentObject;
     currentObject = NULL;
-    PTRACE(6, "SafeColl\tDeleting object (" << objectToDelete << ')');
-    delete objectToDelete;
+    DeleteObject(objectToDelete);
   }
 }
 
 
+void PSafePtrBase::DeleteObject(PSafeObject * obj)
+{
+  if (obj == NULL)
+    return;
+
+  PTRACE(6, "SafeColl\tDeleting object (" << obj << ')');
+  delete obj;
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 
 PSafePtrMultiThreaded::PSafePtrMultiThreaded(PSafeObject * obj, PSafetyMode mode)
   : PSafePtrBase(NULL, mode)
+  , m_objectToDelete(NULL)
 {
-  m_mutex.Wait();
+  LockPtr();
 
   currentObject = obj;
   EnterSafetyMode(WithReference);
 
-  m_mutex.Signal();
+  UnlockPtr();
 }
 
 
@@ -702,13 +742,14 @@ PSafePtrMultiThreaded::PSafePtrMultiThreaded(const PSafeCollection & safeCollect
                                              PSafetyMode mode,
                                              PINDEX idx)
   : PSafePtrBase(NULL, mode)
+  , m_objectToDelete(NULL)
 {
-  m_mutex.Wait();
+  LockPtr();
 
   collection = &safeCollection;
   Assign(idx);
 
-  m_mutex.Signal();
+  UnlockPtr();
 }
 
 
@@ -716,20 +757,22 @@ PSafePtrMultiThreaded::PSafePtrMultiThreaded(const PSafeCollection & safeCollect
                                              PSafetyMode mode,
                                              PSafeObject * obj)
   : PSafePtrBase(NULL, mode)
+  , m_objectToDelete(NULL)
 {
-  m_mutex.Wait();
+  LockPtr();
 
   collection = &safeCollection;
   Assign(obj);
 
-  m_mutex.Signal();
+  UnlockPtr();
 }
 
 
 PSafePtrMultiThreaded::PSafePtrMultiThreaded(const PSafePtrMultiThreaded & enumerator)
+  : m_objectToDelete(NULL)
 {
+  LockPtr();
   enumerator.m_mutex.Wait();
-  m_mutex.Wait();
 
   collection = enumerator.collection;
   currentObject = enumerator.currentObject;
@@ -737,17 +780,17 @@ PSafePtrMultiThreaded::PSafePtrMultiThreaded(const PSafePtrMultiThreaded & enume
 
   EnterSafetyMode(WithReference);
 
-  m_mutex.Signal();
   enumerator.m_mutex.Signal();
+  UnlockPtr();
 }
 
 
 PSafePtrMultiThreaded::~PSafePtrMultiThreaded()
 {
-  m_mutex.Wait();
+  LockPtr();
   ExitSafetyMode(WithDereference);
   currentObject = NULL;
-  m_mutex.Signal();
+  UnlockPtr();
 }
 
 
@@ -760,9 +803,9 @@ PObject::Comparison PSafePtrMultiThreaded::Compare(const PObject & obj) const
 
 void PSafePtrMultiThreaded::SetNULL()
 {
-  m_mutex.Wait();
+  LockPtr();
   PSafePtrBase::SetNULL();
-  m_mutex.Signal();
+  UnlockPtr();
 }
 
 
@@ -775,59 +818,74 @@ PBoolean PSafePtrMultiThreaded::SetSafetyMode(PSafetyMode mode)
 
 void PSafePtrMultiThreaded::Assign(const PSafePtrMultiThreaded & ptr)
 {
+  LockPtr();
   ptr.m_mutex.Wait();
-  m_mutex.Wait();
   PSafePtrBase::Assign(ptr);
-  m_mutex.Signal();
   ptr.m_mutex.Signal();
+  UnlockPtr();
 }
 
 
 void PSafePtrMultiThreaded::Assign(const PSafePtrBase & ptr)
 {
-  m_mutex.Wait();
+  LockPtr();
   PSafePtrBase::Assign(ptr);
-  m_mutex.Signal();
+  UnlockPtr();
 }
 
 
 void PSafePtrMultiThreaded::Assign(const PSafeCollection & safeCollection)
 {
-  m_mutex.Wait();
+  LockPtr();
   PSafePtrBase::Assign(safeCollection);
-  m_mutex.Signal();
+  UnlockPtr();
 }
 
 
 void PSafePtrMultiThreaded::Assign(PSafeObject * obj)
 {
-  m_mutex.Wait();
+  LockPtr();
   PSafePtrBase::Assign(obj);
-  m_mutex.Signal();
+  UnlockPtr();
 }
 
 
 void PSafePtrMultiThreaded::Assign(PINDEX idx)
 {
-  m_mutex.Wait();
+  LockPtr();
   PSafePtrBase::Assign(idx);
-  m_mutex.Signal();
+  UnlockPtr();
 }
 
 
 void PSafePtrMultiThreaded::Next()
 {
-  m_mutex.Wait();
+  LockPtr();
   PSafePtrBase::Next();
-  m_mutex.Signal();
+  UnlockPtr();
 }
 
 
 void PSafePtrMultiThreaded::Previous()
 {
-  m_mutex.Wait();
+  LockPtr();
   PSafePtrBase::Previous();
+  UnlockPtr();
+}
+
+
+void PSafePtrMultiThreaded::DeleteObject(PSafeObject * obj)
+{
+  m_objectToDelete = obj;
+}
+
+
+void PSafePtrMultiThreaded::UnlockPtr()
+{
+  PSafeObject * obj = m_objectToDelete;
+  m_objectToDelete = NULL;
   m_mutex.Signal();
+  PSafePtrBase::DeleteObject(obj);
 }
 
 
