@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26309 $
- * $Author: ededu $
- * $Date: 2011-08-14 18:01:01 -0500 (Sun, 14 Aug 2011) $
+ * $Revision: 28201 $
+ * $Author: rjongbloed $
+ * $Date: 2012-08-14 21:30:31 -0500 (Tue, 14 Aug 2012) $
  */
 
 #include <ptlib.h>
@@ -81,7 +81,8 @@ void CALLBACK CompletionRoutine(DWORD dwError,
 ///////////////////////////////////////////////////////////////////////////////
 // PIPSocket::Address
 
-static int defaultIpAddressFamily = PF_INET;  // PF_UNSPEC;   // default to IPV4
+static int g_defaultIpAddressFamily = PF_INET;  // PF_UNSPEC;   // default to IPV4
+static bool g_suppressCanonicalName = false;
 
 static PIPSocket::Address loopback4(127,0,0,1);
 static PIPSocket::Address broadcast4(INADDR_BROADCAST);
@@ -100,15 +101,27 @@ static PIPSocket::Address any6(16,(const BYTE *)"\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
 #endif
 
 
+void PIPSocket::SetSuppressCanonicalName(bool suppress)
+{
+  g_suppressCanonicalName = suppress;
+}
+
+
+bool PIPSocket::GetSuppressCanonicalName()
+{
+  return g_suppressCanonicalName;
+}
+
+
 int PIPSocket::GetDefaultIpAddressFamily()
 {
-  return defaultIpAddressFamily;
+  return g_defaultIpAddressFamily;
 }
 
 
 void PIPSocket::SetDefaultIpAddressFamily(int ipAdressFamily)
 {
-  defaultIpAddressFamily = ipAdressFamily;
+  g_defaultIpAddressFamily = ipAdressFamily;
 }
 
 
@@ -155,7 +168,7 @@ PBoolean PIPSocket::IsIpAddressFamilyV6Supported()
 PIPSocket::Address PIPSocket::GetDefaultIpAny()
 {
 #if P_HAS_IPV6
-  if (defaultIpAddressFamily != PF_INET)
+  if (g_defaultIpAddressFamily != PF_INET)
     return any6;
 #endif
 
@@ -395,6 +408,8 @@ PIPCacheData::PIPCacheData(struct addrinfo * addr_info, const char * original)
 
   // Fill Host primary informations
   hostname = addr_info->ai_canonname; // Fully Qualified Domain Name (FQDN)
+  if (g_suppressCanonicalName || hostname.IsEmpty())
+    hostname = original;
   if (addr_info->ai_addr != NULL)
     address = PIPSocket::Address(addr_info->ai_family, addr_info->ai_addrlen, addr_info->ai_addr);
 
@@ -543,9 +558,13 @@ PIPCacheData * PHostByName::GetHost(const PString & name)
 #if HAS_GETADDRINFO
 
     struct addrinfo *res = NULL;
-    struct addrinfo hints = { AI_CANONNAME, defaultIpAddressFamily };
+    struct addrinfo hints;
+    memset(&hints, 0, sizeof(hints));
+    if (!g_suppressCanonicalName)
+      hints.ai_flags = AI_CANONNAME;
+    hints.ai_family = g_defaultIpAddressFamily;
     localErrNo = getaddrinfo((const char *)name, NULL , &hints, &res);
-    if (localErrNo != 0 && defaultIpAddressFamily == AF_INET6) {
+    if (localErrNo != 0 && g_defaultIpAddressFamily == AF_INET6) {
       hints.ai_family = AF_INET;
       localErrNo = getaddrinfo((const char *)name, NULL , &hints, &res);
     }
@@ -582,7 +601,7 @@ PIPCacheData * PHostByName::GetHost(const PString & name)
     host_info = Vx_gethostbyname((char *)name, &hostEnt);
     localErrNo = h_errno;
 
-#elif defined P_LINUX
+#elif defined P_LINUX || defined(P_GNU_HURD)
 
     char buffer[REENTRANT_BUFFER_LEN];
     struct hostent hostEnt;
@@ -711,7 +730,7 @@ PIPCacheData * PHostByAddr::GetHost(const PIPSocket::Address & addr)
     struct hostent hostEnt;
     host_info = Vx_gethostbyaddr(addr.GetPointer(), &hostEnt);
 
-#elif defined P_LINUX
+#elif defined P_LINUX || defined(P_GNU_HURD)
 
     char buffer[REENTRANT_BUFFER_LEN];
     struct hostent hostEnt;
@@ -1678,7 +1697,7 @@ PBoolean PIPSocket::Address::IsAny() const
 PIPSocket::Address::Address()
 {
 #if P_HAS_IPV6
-  if(defaultIpAddressFamily == AF_INET6)
+  if (g_defaultIpAddressFamily == AF_INET6)
     *this = loopback6;
   else
 #endif
@@ -2134,30 +2153,48 @@ PBoolean PIPSocket::Address::IsRFC1918() const
 }
 
 PIPSocket::InterfaceEntry::InterfaceEntry()
-  : ipAddr(GetDefaultIpAny())
-  , netMask(GetDefaultIpAny())
+  : m_ipAddress(0, NULL)
+  , m_netMask(0, NULL)
 {
 }
 
-PIPSocket::InterfaceEntry::InterfaceEntry(const PString & _name,
-                                          const Address & _addr,
-                                          const Address & _mask,
-                                          const PString & _macAddr)
-  : name(_name.Trim()),
-    ipAddr(_addr),
-    netMask(_mask),
-    macAddr(_macAddr)
+PIPSocket::InterfaceEntry::InterfaceEntry(const PString & name,
+                                          const Address & address,
+                                          const Address & mask,
+                                          const PString & macAddress)
+  : m_name(name.Trim())
+  , m_ipAddress(address)
+  , m_netMask(mask)
+  , m_macAddress(macAddress)
 {
+  SanitiseName(m_name);
+}
+
+
+void PIPSocket::InterfaceEntry::SanitiseName(PString & name)
+{
+  /* HACK!!
+     At various points in PTLib (and OPAL) the interface name is used in
+     situations where there can be confusion in parsing. For example a URL can
+     be http:://[::%interface]:2345 and a ] in interface name blows it to
+     pieces. Similarly, "ip:port" style description which can be used a LOT,
+     will also fail. At this late stage it is too hard to change all the other
+     places, so we hack the fairly rare cases by translating those special
+     characters.
+   */ 
+  name.Replace('[', '{', true);
+  name.Replace(']', '}', true);
+  name.Replace(':', ';', true);
 }
 
 
 void PIPSocket::InterfaceEntry::PrintOn(ostream & strm) const
 {
-  strm << ipAddr;
-  if (!macAddr)
-    strm << " <" << macAddr << '>';
-  if (!name)
-    strm << " (" << name << ')';
+  strm << m_ipAddress;
+  if (!m_macAddress)
+    strm << " <" << m_macAddress << '>';
+  if (!m_name)
+    strm << " (" << m_name << ')';
 }
 
 
