@@ -23,9 +23,9 @@
  *
  * Contributor(s): Mark Cooke (mpc@star.sr.bham.ac.uk)
  *
- * $Revision: 24328 $
- * $Author: ededu $
- * $Date: 2010-05-07 11:20:51 -0500 (Fri, 07 May 2010) $
+ * $Revision: 25752 $
+ * $Author: rjongbloed $
+ * $Date: 2011-05-11 21:50:32 -0500 (Wed, 11 May 2011) $
  */
 
 #ifdef __GNUC__
@@ -83,7 +83,7 @@ ostream & operator<<(ostream & strm, PVideoDevice::VideoFormat fmt)
 static struct {
   const char * colourFormat;
   unsigned     bitsPerPixel;
-} colourFormatBPPTab[] = {
+} ColourFormatBPPTab[] = {
   { "YUV420P", 12 },
   { "I420",    12 },
   { "IYUV",    12 },
@@ -106,7 +106,9 @@ static struct {
   { "GreyF",    8 },
   { "UYVY422", 16 },
   { "UYV444",  24 },
-  { "SBGGR8",   8 }
+  { "SBGGR8",   8 },
+  { "JPEG",    24 },
+  { "MJPEG",   24 }
 };
 
 
@@ -164,6 +166,21 @@ static VideoDevice * CreateDeviceWithDefaults(PString & adjustedDeviceName,
 ///////////////////////////////////////////////////////////////////////////////
 // PVideoDevice
 
+ostream & operator<<(ostream & strm, PVideoFrameInfo::ResizeMode mode)
+{
+  switch (mode) {
+    case PVideoFrameInfo::eScale :
+      return strm << "Scaled";
+    case PVideoFrameInfo::eCropCentre :
+      return strm << "Centred";
+    case PVideoFrameInfo::eCropTopLeft :
+      return strm << "Cropped";
+    default :
+      return strm << "ResizeMode<" << (int)mode << '>';
+  }
+}
+
+
 PVideoFrameInfo::PVideoFrameInfo()
   : frameWidth(CIFWidth)
   , frameHeight(CIFHeight)
@@ -174,6 +191,35 @@ PVideoFrameInfo::PVideoFrameInfo()
 }
 
 
+PVideoFrameInfo::PVideoFrameInfo(unsigned        width,
+                                 unsigned        height,
+                                 const PString & format,
+                                 unsigned        rate,
+                                 ResizeMode      resize)
+  : frameWidth(width)
+  , frameHeight(height)
+  , frameRate(rate)
+  , colourFormat(format)
+  , resizeMode(resize)
+{
+}
+
+
+void PVideoFrameInfo::PrintOn(ostream & strm) const
+{
+  if (!colourFormat.IsEmpty())
+    strm << colourFormat << ':';
+
+  strm << AsString(frameWidth, frameHeight);
+
+  if (frameRate > 0)
+    strm << '@' << frameRate;
+
+  if (resizeMode < eMaxResizeMode)
+    strm << '/' << resizeMode;
+}
+
+
 PBoolean PVideoFrameInfo::SetFrameSize(unsigned width, unsigned height)
 {
   if (width < 8 || height < 8)
@@ -207,6 +253,38 @@ unsigned PVideoFrameInfo::GetFrameHeight() const
   return h;
 }
 
+PBoolean PVideoFrameInfo::SetFrameSar(unsigned width, unsigned height)
+{
+    if(height == 0 || width == 0)
+    {
+        return PFalse;
+    }
+  sarWidth  = width;
+  sarHeight = height;
+  return PTrue;
+}
+
+PBoolean PVideoFrameInfo::GetSarSize(unsigned & width, unsigned & height) const
+{
+  width  = sarWidth;
+  height = sarHeight;
+  return PTrue;
+}
+
+unsigned PVideoFrameInfo::GetSarWidth() const
+{
+  unsigned w,h;
+  GetSarSize(w, h);
+  return w;
+}
+
+
+unsigned PVideoFrameInfo::GetSarHeight() const
+{
+  unsigned w,h;
+  GetSarSize(w, h);
+  return h;
+}
 
 PBoolean PVideoFrameInfo::SetFrameRate(unsigned rate)
 {
@@ -231,8 +309,8 @@ PBoolean PVideoFrameInfo::SetColourFormat(const PString & colourFmt)
     return PTrue;
   }
 
-  for (PINDEX i = 0; i < PARRAYSIZE(colourFormatBPPTab); i++) {
-    if (SetColourFormat(colourFormatBPPTab[i].colourFormat))
+  for (PINDEX i = 0; i < PARRAYSIZE(ColourFormatBPPTab); i++) {
+    if (SetColourFormat(ColourFormatBPPTab[i].colourFormat))
       return PTrue;
   }
 
@@ -249,58 +327,120 @@ const PString & PVideoFrameInfo::GetColourFormat() const
 PINDEX PVideoFrameInfo::CalculateFrameBytes(unsigned width, unsigned height,
                                               const PString & colourFormat)
 {
-  for (PINDEX i = 0; i < PARRAYSIZE(colourFormatBPPTab); i++) {
-    if (colourFormat *= colourFormatBPPTab[i].colourFormat)
-      return width * height * colourFormatBPPTab[i].bitsPerPixel/8;
+  for (PINDEX i = 0; i < PARRAYSIZE(ColourFormatBPPTab); i++) {
+    if (colourFormat *= ColourFormatBPPTab[i].colourFormat)
+      return width * height * ColourFormatBPPTab[i].bitsPerPixel/8;
   }
   return 0;
 }
  
 
-PBoolean PVideoFrameInfo::ParseSize(const PString & str, unsigned & width, unsigned & height)
+bool PVideoFrameInfo::Parse(const PString & str)
 {
-  static struct {
-    const char * name;
-    unsigned width;
-    unsigned height;
-  } const sizeTable[] = {
-      { "cif",    PVideoDevice::CIFWidth,   PVideoDevice::CIFHeight   },
-      { "qcif",   PVideoDevice::QCIFWidth,  PVideoDevice::QCIFHeight  },
-      { "sqcif",  PVideoDevice::SQCIFWidth, PVideoDevice::SQCIFHeight },
-      { "cif4",   PVideoDevice::CIF4Width,  PVideoDevice::CIF4Height  },
-      { "4cif",   PVideoDevice::CIF4Width,  PVideoDevice::CIF4Height  },
-      { "cif16",  PVideoDevice::CIF16Width, PVideoDevice::CIF16Height },
-      { "16cif",  PVideoDevice::CIF16Width, PVideoDevice::CIF16Height },
+  PString newFormat = colourFormat;
+  PINDEX formatOffset = str.Find(':');
+  if (formatOffset == 0)
+    return false;
 
-      { "ccir601",720,                      486                       },
-      { "ntsc",   720,                      480                       },
-      { "pal",    768,                      576                       },
-      { "hdtvp",  1280,                     720                       },
-      { "hd720",  1280,                     720                       },
-      { "hdtvi",  1920,                     1080                      },
-      { "hd1080", 1920,                     1080                      },
+  if (formatOffset == P_MAX_INDEX)
+    formatOffset = 0;
+  else
+    newFormat = str.Left(formatOffset++);
 
-      { "cga",    320,                      240                       },
-      { "vga",    640,                      480                       },
-      { "wvga",   854,                      480                       },
-      { "svga",   800,                      600                       },
-      { "xga",    1024,                     768                       },
-      { "sxga",   1280,                     1024                      },
-      { "wsxga",  1440,                     900                       },
-      { "sxga+",  1400,                     1050                      },
-      { "wsxga+", 1680,                     1050                      },
-      { "uxga",   1600,                     1200                      },
-      { "wuxga",  1920,                     1200                      },
-      { "qxga",   2048,                     1536                      },
-      { "wqxga",  2560,                     1600                      },
-      { }
-  };
 
-  for (int i = 0; sizeTable[i].name != NULL; i++) {
-    if (str *= sizeTable[i].name) {
-      width = sizeTable[i].width;
-      height = sizeTable[i].height;
-      return PTrue;
+  ResizeMode newMode = resizeMode;
+  PINDEX resizeOffset = str.Find('/', formatOffset);
+  if (resizeOffset != P_MAX_INDEX) {
+    static struct {
+      const char * name;
+      ResizeMode   mode;
+    } const ResizeNames[] = {
+      { "scale",   eScale },
+      { "resize",  eScale },
+      { "scaled",  eScale },
+      { "centre",  eCropCentre },
+      { "centred", eCropCentre },
+      { "center",  eCropCentre },
+      { "centered",eCropCentre },
+      { "crop",    eCropTopLeft },
+      { "cropped", eCropTopLeft },
+      { "topleft", eCropTopLeft }
+    };
+
+    PCaselessString crop = str.Mid(resizeOffset+1);
+    PINDEX resizeIndex = 0;
+    while (crop != ResizeNames[resizeIndex].name) {
+      if (++resizeIndex >= PARRAYSIZE(ResizeNames))
+        return false;
+    }
+    newMode = ResizeNames[resizeIndex].mode;
+  }
+
+
+  int newRate = frameRate;
+  PINDEX rateOffset = str.Find('@', formatOffset);
+  if (rateOffset == P_MAX_INDEX)
+    rateOffset = resizeOffset;
+  else {
+    newRate = str.Mid(rateOffset+1).AsInteger();
+    if (newRate < 1 || newRate > 100)
+      return false;
+  }
+
+  if (!ParseSize(str(formatOffset, rateOffset-1), frameWidth, frameHeight))
+    return false;
+
+  colourFormat = newFormat;
+  frameRate = newRate;
+  resizeMode = newMode;
+  return true;
+}
+
+
+static struct {
+  const char * name;
+  unsigned width;
+  unsigned height;
+} const SizeTable[] = {
+    { "CIF",    PVideoDevice::CIFWidth,   PVideoDevice::CIFHeight   },
+    { "QCIF",   PVideoDevice::QCIFWidth,  PVideoDevice::QCIFHeight  },
+    { "SQCIF",  PVideoDevice::SQCIFWidth, PVideoDevice::SQCIFHeight },
+    { "CIF4",   PVideoDevice::CIF4Width,  PVideoDevice::CIF4Height  },
+    { "4CIF",   PVideoDevice::CIF4Width,  PVideoDevice::CIF4Height  },
+    { "CIF16",  PVideoDevice::CIF16Width, PVideoDevice::CIF16Height },
+    { "16CIF",  PVideoDevice::CIF16Width, PVideoDevice::CIF16Height },
+
+    { "CCIR601",720,                      486                       },
+    { "NTSC",   720,                      480                       },
+    { "PAL",    768,                      576                       },
+    { "HD480",  PVideoDevice::HD480Width, PVideoDevice::HD480Height },
+    { "HDTVP",  PVideoDevice::HD720Width, PVideoDevice::HD720Height },
+    { "HD720",  PVideoDevice::HD720Width, PVideoDevice::HD720Height },
+    { "HDTVI",  PVideoDevice::HD1080Width,PVideoDevice::HD1080Height},
+    { "HD1080", PVideoDevice::HD1080Width,PVideoDevice::HD1080Height},
+
+    { "CGA",    320,                      240                       },
+    { "VGA",    640,                      480                       },
+    { "WVGA",   854,                      480                       },
+    { "SVGA",   800,                      600                       },
+    { "XGA",    1024,                     768                       },
+    { "SXGA",   1280,                     1024                      },
+    { "WSXGA",  1440,                     900                       },
+    { "SXGA+",  1400,                     1050                      },
+    { "WSXGA+", 1680,                     1050                      },
+    { "UXGA",   1600,                     1200                      },
+    { "WUXGA",  1920,                     1200                      },
+    { "QXGA",   2048,                     1536                      },
+    { "WQXGA",  2560,                     1600                      },
+};
+
+bool PVideoFrameInfo::ParseSize(const PString & str, unsigned & width, unsigned & height)
+{
+  for (int i = 0; i < PARRAYSIZE(SizeTable); i++) {
+    if (str *= SizeTable[i].name) {
+      width = SizeTable[i].width;
+      height = SizeTable[i].height;
+      return true;
     }
   }
 
@@ -308,6 +448,26 @@ PBoolean PVideoFrameInfo::ParseSize(const PString & str, unsigned & width, unsig
 }
 
 
+PString PVideoFrameInfo::AsString(unsigned width, unsigned height)
+{
+  for (int i = 0; i < PARRAYSIZE(SizeTable); i++) {
+    if (SizeTable[i].width == width && SizeTable[i].height == height)
+      return SizeTable[i].name;
+  }
+
+  return psprintf("%ux%u", width, height);
+}
+
+
+PStringArray PVideoFrameInfo::GetSizeNames()
+{
+  PStringArray names(PARRAYSIZE(SizeTable));
+  for (int i = 0; i < PARRAYSIZE(SizeTable); i++)
+    names[i] = SizeTable[i].name;
+  return names;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 // PVideoDevice
 
@@ -496,103 +656,77 @@ int PVideoDevice::GetChannel() const
 
 PBoolean PVideoDevice::SetColourFormatConverter(const PString & newColourFmt)
 {
+  if (converter != NULL) {
+    if (CanCaptureVideo()) {
+      if (converter->GetDstColourFormat() == newColourFmt)
+        return true;
+    }
+    else {
+      if (converter->GetSrcColourFormat() == newColourFmt)
+        return true;
+    }
+  }
+  else {
+    if (colourFormat == newColourFmt)
+      return true;
+  }
+
+  PString newColourFormat = newColourFmt; // make copy, just in case newColourFmt is reference to member colourFormat
+
+  if (!SetColourFormat(newColourFormat) &&
+        (preferredColourFormat.IsEmpty() || !SetColourFormat(preferredColourFormat))) {
+    /************************
+      Eventually, need something more sophisticated than this, but for the
+      moment pick the known colour formats that the device is very likely to
+      support and then look for a conversion routine from that to the
+      destination format.
+
+      What we really want is some sort of better heuristic that looks at
+      computational requirements of each converter and picks a pair of formats
+      that the hardware supports and uses the least CPU.
+    */
+
+    PINDEX knownFormatIdx = 0;
+    while (!SetColourFormat(ColourFormatBPPTab[knownFormatIdx].colourFormat)) {
+      if (++knownFormatIdx >= PARRAYSIZE(ColourFormatBPPTab)) {
+        PTRACE(2, "PVidDev\tSetColourFormatConverter FAILED for " << newColourFormat);
+        return false;
+      }
+    }
+  }
+
+  PTRACE(3, "PVidDev\tSetColourFormatConverter success for native " << colourFormat);
+
   PVideoFrameInfo src = *this;
   PVideoFrameInfo dst = *this;
 
-  PString colourFmt = newColourFmt; // make copy, just in case newColourFmt is reference to member colourFormat
-
   if (converter != NULL) {
-    if (CanCaptureVideo()) {
-      if (converter->GetDstColourFormat() == colourFmt)
-        return PTrue;
-    }
-    else {
-      if (converter->GetSrcColourFormat() == colourFmt)
-        return PTrue;
-    }
     converter->GetSrcFrameInfo(src);
     converter->GetDstFrameInfo(dst);
     delete converter;
     converter = NULL;
   }
-  
-  if (!preferredColourFormat.IsEmpty()) {
-    PTRACE(4,"PVidDev\tSetColourFormatConverter, want " << colourFmt << " trying " << preferredColourFormat);
-    if (SetColourFormat(preferredColourFormat)) {
-      if (CanCaptureVideo()) {
-        PTRACE(4,"PVidDev\tSetColourFormatConverter set camera to native "<< preferredColourFormat);
-        if (preferredColourFormat != colourFmt)
-          src.SetColourFormat(preferredColourFormat);
-      }
-      else {
-        PTRACE(4,"PVidDev\tSetColourFormatConverter set renderer to "<< preferredColourFormat);
-        if (preferredColourFormat != colourFmt)
-          dst.SetColourFormat(preferredColourFormat);
-      }
 
-      if (nativeVerticalFlip || src.GetColourFormat() != dst.GetColourFormat()) {
-        converter = PColourConverter::Create(src, dst);
-        if (converter != NULL) {
-          converter->SetVFlipState(nativeVerticalFlip);
-          return PTrue;
-        }
-      }
+  if (nativeVerticalFlip || colourFormat != newColourFormat) {
+    if (CanCaptureVideo()) {
+      src.SetColourFormat(colourFormat);
+      dst.SetColourFormat(newColourFormat);
     }
-  }
-  
-  if (SetColourFormat(colourFmt)) {
-    if (nativeVerticalFlip) {
-      src.SetColourFormat(colourFmt);
-      dst.SetColourFormat(colourFmt);
-      converter = PColourConverter::Create(src, dst);
-      if (PAssertNULL(converter) == NULL)
-        return PFalse;
-      converter->SetVFlipState(nativeVerticalFlip);
+    else {
+      src.SetColourFormat(newColourFormat);
+      dst.SetColourFormat(colourFormat);
     }
 
-    PTRACE(3, "PVidDev\tSetColourFormatConverter success for native " << colourFmt);    
-    return PTrue;
-  }
-  
-  /************************
-    Eventually, need something more sophisticated than this, but for the
-    moment pick the known colour formats that the device is very likely to
-    support and then look for a conversion routine from that to the
-    destination format.
+    converter = PColourConverter::Create(src, dst);
+    if (converter == NULL) {
+      PTRACE(2, "PVidDev\tSetColourFormatConverter failed to crate converter from " << src << " to " << dst);
+      return false;
+    }
 
-    What we really want is some sort of better heuristic that looks at
-    computational requirements of each converter and picks a pair of formats
-    that the hardware supports and uses the least CPU.
-  */
-
-  PINDEX knownFormatIdx = 0;
-  while (knownFormatIdx < PARRAYSIZE(colourFormatBPPTab)) {
-    PString formatToTry = colourFormatBPPTab[knownFormatIdx].colourFormat;
-    PTRACE(4,"PVidDev\tSetColourFormatConverter, want " << colourFmt << " trying " << formatToTry);
-    if (SetColourFormat(formatToTry)) {
-      if (CanCaptureVideo()) {
-        PTRACE(4,"PVidDev\tSetColourFormatConverter set camera to "<< formatToTry);
-        src.SetColourFormat(formatToTry);
-        dst.SetColourFormat(colourFmt);
-      }
-      else {
-        PTRACE(4,"PVidDev\tSetColourFormatConverter set renderer to "<< formatToTry);
-        dst.SetColourFormat(formatToTry);
-        src.SetColourFormat(colourFmt);
-      }
-      converter = PColourConverter::Create(src, dst);
-      if (converter != NULL) {
-        // set converter properties that depend on this color format
-        PTRACE(3, "PVidDev\tSetColourFormatConverter succeeded for " << colourFmt << " and device using " << formatToTry);
-        converter->SetVFlipState(nativeVerticalFlip);
-        return PTrue;
-      } 
-    } 
-    knownFormatIdx++;
+    converter->SetVFlipState(nativeVerticalFlip);
   }
 
-  PTRACE(2, "PVidDev\tSetColourFormatConverter  FAILED for " << colourFmt);
-  return PFalse;
+  return true;
 }
 
 
@@ -631,53 +765,6 @@ PBoolean PVideoDevice::GetFrameSizeLimits(unsigned & minWidth,
 }
 
 
-static struct {
-    unsigned asked_width, asked_height, device_width, device_height;
-} framesizeTab[] = {    
-    { 704, 576,    640, 480 },
-    { 640, 480,    704, 576 },
-    { 640, 480,    352, 288 },
-
-    { 352, 288,    704, 576 },
-    { 352, 288,    640, 480 },
-    { 352, 288,    352, 240 },
-    { 352, 288,    320, 240 },
-    { 352, 288,    176, 144 },
-    { 352, 288,   1024, 576 }, /* High resolution need to be set at the end */
-    { 352, 288,   1280, 960 },
-
-    { 352, 240,    352, 288 },
-    { 352, 240,    320, 240 },
-    { 352, 240,    640, 480 },
-
-    { 320, 240,    352, 288 },
-    { 320, 240,    352, 240 },
-    { 320, 240,    640, 480 },
-
-    { 176, 144,    352, 288 },
-    { 176, 144,    352, 240 },
-    { 176, 144,    320, 240 },
-    { 176, 144,    176, 120 },
-    { 176, 144,    160, 120 },
-    { 176, 144,    640, 480 },
-    { 176, 144,   1024, 576 },
-    { 176, 144,   1280, 960 }, /* High resolution need to be set at the end */
-
-    { 176, 120,    352, 288 },
-    { 176, 120,    352, 240 },
-    { 176, 120,    320, 240 },
-    { 176, 120,    176, 144 },
-    { 176, 120,    160, 120 },
-    { 176, 120,    640, 480 },
-
-    { 160, 120,    352, 288 },
-    { 160, 120,    352, 240 },
-    { 160, 120,    320, 240 },
-    { 160, 120,    176, 144 },
-    { 160, 120,    176, 120 },
-    { 160, 120,    640, 480 },
-};
-
 PBoolean PVideoDevice::SetFrameSizeConverter(unsigned width, unsigned height, ResizeMode resizeMode)
 {
   if (SetFrameSize(width, height)) {
@@ -694,18 +781,9 @@ PBoolean PVideoDevice::SetFrameSizeConverter(unsigned width, unsigned height, Re
   }
 
   // Try and get the most compatible physical frame size to convert from/to
-  PINDEX i;
-  for (i = 0; i < PARRAYSIZE(framesizeTab); i++) {
-    if (framesizeTab[i].asked_width == width && framesizeTab[i].asked_height == height &&
-        SetFrameSize(framesizeTab[i].device_width, framesizeTab[i].device_height))
-      break;
-  }
-  if (i >= PARRAYSIZE(framesizeTab)) {
-    // Failed to find a resolution the device can do so far, so try
-    // using the maximum width and height it claims it can do.
-    unsigned minWidth, minHeight, maxWidth, maxHeight;
-    if (GetFrameSizeLimits(minWidth, minHeight, maxWidth, maxHeight))
-      SetFrameSize(maxWidth, maxHeight);
+  if (!SetNearestFrameSize(width, height)) {
+    PTRACE(1, "PVidDev\tCannot set an apropriate size to scale from.");
+    return false;
   }
 
   // Now create the converter ( if not already exist)
@@ -738,6 +816,25 @@ PBoolean PVideoDevice::SetFrameSizeConverter(unsigned width, unsigned height, Re
 }
 
 
+PBoolean PVideoDevice::SetNearestFrameSize(unsigned width, unsigned height)
+{
+  unsigned minWidth, minHeight, maxWidth, maxHeight;
+  if (GetFrameSizeLimits(minWidth, minHeight, maxWidth, maxHeight)) {
+    if (width < minWidth)
+      width = minWidth;
+    else if (width > maxWidth)
+      width = maxWidth;
+
+    if (height < minHeight)
+      height = minHeight;
+    else if (height > maxHeight)
+      height = maxHeight;
+  }
+
+  return SetFrameSize(width, height);
+}
+
+
 PBoolean PVideoDevice::SetFrameSize(unsigned width, unsigned height)
 {
 #if PTRACING
@@ -745,32 +842,18 @@ PBoolean PVideoDevice::SetFrameSize(unsigned width, unsigned height)
   unsigned oldHeight = frameHeight;
 #endif
 
-  unsigned minWidth, minHeight, maxWidth, maxHeight;
-  GetFrameSizeLimits(minWidth, minHeight, maxWidth, maxHeight);
-
-  if (width < minWidth)
-    frameWidth = minWidth;
-  else if (width > maxWidth)
-    frameWidth = maxWidth;
-  else
-    frameWidth = width;
-
-  if (height < minHeight)
-    frameHeight = minHeight;
-  else if (height > maxHeight)
-    frameHeight = maxHeight;
-  else
-    frameHeight = height;
+  frameWidth = width;
+  frameHeight = height;
 
   if (converter != NULL) {
-    if (!converter->SetSrcFrameSize(width, height) ||
-        !converter->SetDstFrameSize(width, height)) {
+    if ((!converter->SetSrcFrameSize(width, height)) ||
+        (!converter->SetDstFrameSize(width, height))) {
       PTRACE(1, "PVidDev\tSetFrameSize with converter failed with " << width << 'x' << height);
       return PFalse;
     }
   }
 
-  PTRACE_IF(2, oldWidth != frameWidth || oldHeight != frameHeight,
+  PTRACE_IF(3, oldWidth != frameWidth || oldHeight != frameHeight,
             "PVidDev\tSetFrameSize to " << frameWidth << 'x' << frameHeight);
   return PTrue;
 }
@@ -908,6 +991,25 @@ PString PVideoControlInfo::AsString(const InputControlType & ctype)
 	return PString();
 }
 
+////////////////////////////////////////////////////////////////////////////////////////////
+
+PString PVideoInteractionInfo::AsString(const InputInteractType & ctype)
+{
+	switch (ctype) {
+		case InteractKey:
+			return "Remote Key Press";
+		case InteractMouse:
+			return "Remote Mouse Move/Click";
+		case InteractNavigate:
+			return "Remote Navigation";
+		case InteractRTSP:
+			return "Remote RTSP Commands";
+		case InteractOther:
+			return "Custom/Other";
+	}
+	return PString();
+}
+
 /////////////////////////////////////////////////////////////////////////////////////////////
 
 PVideoInputControl::~PVideoInputControl()  
@@ -1042,6 +1144,12 @@ PBoolean PVideoOutputDevice::GetPosition(int &, int &) const
 }
 
 
+bool PVideoOutputDevice::SetPosition(int, int)
+{
+  return false;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 // PVideoOutputDeviceRGB
 
@@ -1094,6 +1202,9 @@ PBoolean PVideoOutputDeviceRGB::SetFrameSize(unsigned width, unsigned height)
 {
   PWaitAndSignal m(mutex);
 
+  if (frameWidth == width && frameHeight == height)
+    return true;
+
   if (!PVideoOutputDevice::SetFrameSize(width, height))
     return PFalse;
 
@@ -1116,7 +1227,7 @@ PBoolean PVideoOutputDeviceRGB::SetFrameData(unsigned x, unsigned y,
 {
   PWaitAndSignal m(mutex);
 
-  if (x+width > frameWidth || y+height > frameHeight)
+  if (x+width > frameWidth || y+height > frameHeight || PAssertNULL(data) == NULL)
     return PFalse;
 
   if (x == 0 && width == frameWidth && y == 0 && height == frameHeight) {
@@ -1280,6 +1391,10 @@ PBoolean PVideoInputDevice::GetDeviceCapabilities(const PString & deviceName, co
   return pluginMgr->GetPluginsDeviceCapabilities(videoInputPluginBaseClass,driverName,deviceName, caps);
 }
 
+PVideoInputControl * PVideoInputDevice::GetVideoInputControls()
+{
+	return NULL;
+}
 
 
 PVideoInputDevice * PVideoInputDevice::CreateOpenedDevice(const PString & driverName,
@@ -1315,6 +1430,75 @@ PVideoInputDevice * PVideoInputDevice::CreateOpenedDevice(const OpenArgs & args,
   return NULL;
 }
 
+
+PBoolean PVideoInputDevice::SetNearestFrameSize(unsigned width, unsigned height)
+{
+  if (PVideoDevice::SetNearestFrameSize(width, height))
+    return true;
+
+  // Get the discrete sizes grabber is capable of
+  Capabilities caps;
+  if (!GetDeviceCapabilities(&caps))
+    return false;
+
+  // First try and pick one with the same width
+  std::list<PVideoFrameInfo>::iterator it;
+  for (it = caps.framesizes.begin(); it != caps.framesizes.end(); ++it) {
+    if (it->GetFrameWidth() == width)
+      return SetFrameSize(width, it->GetFrameHeight());
+  }
+
+  // Then try for the same height
+  for (it = caps.framesizes.begin(); it != caps.framesizes.end(); ++it) {
+    if (it->GetFrameHeight() == height)
+      return SetFrameSize(it->GetFrameWidth(), height);
+  }
+
+  // Then try for double the size
+  for (it = caps.framesizes.begin(); it != caps.framesizes.end(); ++it) {
+    unsigned w, h;
+    it->GetFrameSize(w, h);
+    if (w == width*2 && h == height*2)
+      return SetFrameSize(w, h);
+  }
+
+  // Then try for half the size
+  for (it = caps.framesizes.begin(); it != caps.framesizes.end(); ++it) {
+    unsigned w, h;
+    it->GetFrameSize(w, h);
+    if (w == width/2 && h == height/2)
+      return SetFrameSize(w, h);
+  }
+
+  // Now try and pick one that has the nearest number of pixels in total.
+  unsigned pixels = width*height;
+  unsigned widthToUse = 0, heightToUse = 0;
+  int diff = INT_MAX;
+  for (it = caps.framesizes.begin(); it != caps.framesizes.end(); ++it) {
+    unsigned w, h;
+    it->GetFrameSize(w, h);
+    int d = w*h - pixels;
+    if (d < 0)
+      d = -d;
+    if (diff > d && 
+          // Ensure we don't pick one dimension greater and one 
+          // lower because we can't shrink one and grow the other.
+          ((w < width && h < height) || 
+	       (w > width && h > height))
+    ) {
+      diff = d;
+      widthToUse = w;
+      heightToUse = h;
+    }
+  }
+
+  if (widthToUse == 0)
+    return false;
+
+  return SetFrameSize(widthToUse, heightToUse);
+}
+
+
 PBoolean PVideoInputDevice::GetFrame(PBYTEArray & frame)
 {
   PINDEX returned;
@@ -1345,6 +1529,24 @@ PBoolean PVideoInputDevice::GetFrameDataNoDelay(
   return GetFrameDataNoDelay(buffer, bytesReturned);
 }
 
+bool PVideoInputDevice::FlowControl(const void * /*flowData*/)
+{
+    return false;
+}
+
+
+bool PVideoInputDevice::SetCaptureMode(unsigned)
+{
+  return false;
+}
+
+
+int PVideoInputDevice::GetCaptureMode() const
+{
+  return -1;
+}
+
+
 PBoolean PVideoOutputDevice::SetFrameData(
       unsigned x,
       unsigned y,
@@ -1358,8 +1560,26 @@ PBoolean PVideoOutputDevice::SetFrameData(
   return SetFrameData(x, y, width, height, data, endFrame);
 }
 
+PBoolean PVideoOutputDevice::SetFrameData(
+      unsigned x,
+      unsigned y,
+      unsigned width,
+      unsigned height,
+      unsigned /*sarwidth*/,
+      unsigned /*sarheight*/,
+      const BYTE * data,
+      PBoolean endFrame,
+      unsigned flags,
+	  const void * /*mark*/
+)
+{
+  return SetFrameData(x, y, width, height, data, endFrame, flags);
+}
 
-
+PBoolean PVideoOutputDevice::DisableDecode() 
+{
+	return false;
+}
 
 
 ////////////////////////////////////////////////////////////////////////////////////////////
