@@ -30,9 +30,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26313 $
- * $Author: rjongbloed $
- * $Date: 2011-08-14 22:11:45 -0500 (Sun, 14 Aug 2011) $
+ * $Revision: 27570 $
+ * $Author: ededu $
+ * $Date: 2012-05-03 10:00:22 -0500 (Thu, 03 May 2012) $
  */
 
 
@@ -87,18 +87,20 @@
 
 #else // _WIN32_WCE
 
+  /* workaround a compile error with mingw-w64 on sprintf member function
+     below. Even though the member function is not the same thing as the
+     global function which _is_ deprecated.
+
+     Also applies to a warning in MSVC.
+
+     Need to define this before dshow.h inclusion.
+   */
+  #define STRSAFE_NO_DEPRECATE
+
   #include <dshow.h>
   #include <initguid.h>
-
-#ifdef __MINGW32__
   #include <ks.h>
   #include <ksmedia.h>
-#else
-  #pragma warning(disable:4201)
-  #include <Ks.h>
-  #include <KsMedia.h>
-  #pragma warning(default:4201)
-#endif
 
   class PVideoInputControl_DirectShow : public PVideoInputControl
   {
@@ -237,6 +239,7 @@ class PVideoInputDevice_DirectShow : public PVideoInputDevice
     CComPtr<ICaptureGraphBuilder2> m_pCaptureBuilder;
     CComPtr<IBaseFilter>           m_pCaptureFilter;
     CComPtr<IPin>                  m_pCameraOutPin; // Camera output out -> Transform Input pin
+    GUID                           m_selectedGUID;
 
 #ifdef _WIN32_WCE
     PSampleGrabber               * m_pSampleGrabber;
@@ -279,15 +282,15 @@ static struct {
     { "UYVY422", MEDIASUBTYPE_UYVY   }
 };
 
-static PString GUID2Format(GUID m_guid)
+static PString GUID2Format(GUID guid)
 {
    for (int j = 0; j < sizeof(ColourFormat2GUID)/sizeof(ColourFormat2GUID[0]); j++) {
-    if (m_guid == ColourFormat2GUID[j].m_guid)
+    if (guid == ColourFormat2GUID[j].m_guid)
       return ColourFormat2GUID[j].m_colourFormat;
    }
 
    wchar_t guidName[256];
-   if (StringFromGUID2(m_guid, guidName, sizeof(guidName)) <= 0)
+   if (StringFromGUID2(guid, guidName, sizeof(guidName)) <= 0)
        return "UNKNOWN"; // Can't use this entry!
 
    return guidName;
@@ -317,7 +320,6 @@ static PString ErrorMessage(HRESULT hr)
     return msg;
 #endif
 
-#pragma warning(disable:4995)
   char hex[20];
   _snprintf(hex, sizeof(hex), "0x%08x", hr);
   return hex;
@@ -629,6 +631,7 @@ bool PVideoInputDevice_DirectShow::SetPinFormat(unsigned useDefaultColourOrSize)
         }
       }
 
+      m_selectedGUID = pMediaFormat->subtype;
       m_maxFrameBytes = CalculateFrameBytes(frameWidth, frameHeight, colourFormat);
 
       if (running)
@@ -1642,7 +1645,7 @@ bool PVideoInputDevice_DirectShow::PlatformOpen()
   AM_MEDIA_TYPE mt;
   ZeroMemory(&mt, sizeof(AM_MEDIA_TYPE));
   mt.majortype = MEDIATYPE_Video;
-  mt.subtype = MEDIASUBTYPE_RGB24;
+  mt.subtype = m_selectedGUID;
   CHECK_ERROR_RETURN(m_pSampleGrabber->SetMediaType(&mt));
 
   CHECK_ERROR_RETURN(m_pSampleGrabber->SetBufferSamples(true));
