@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
- * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Revision: 24461 $
+ * $Author: shorne $
+ * $Date: 2010-06-06 09:06:37 -0500 (Sun, 06 Jun 2010) $
  */
 
 #include <ptlib.h>
@@ -96,6 +96,9 @@ PBoolean PMail::LogOn(const PString & username,
   return LogOnCommonInterface(username, password, service);
 }
 
+#ifdef _WIN64
+  #pragma warning(disable:4100)
+#endif
 
 PBoolean PMail::LogOnCommonInterface(const char * username,
                                  const char * password,
@@ -736,6 +739,9 @@ PMail::LookUpResult PMail::LookUp(const PString & name, PString * fullName)
   return LookUpError;
 }
 
+#ifdef _WIN64
+  #pragma warning(default:4100)
+#endif
 
 int PMail::GetErrorCode() const
 {
