@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26448 $
+ * $Revision: 28050 $
  * $Author: rjongbloed $
- * $Date: 2011-09-19 22:01:53 -0500 (Mon, 19 Sep 2011) $
+ * $Date: 2012-07-18 02:15:30 -0500 (Wed, 18 Jul 2012) $
  */
 
 #include <ptlib.h>
@@ -83,6 +83,7 @@ enum {
   SvcCmdAutoRestart,
   SvcCmdNoRestart,
   SvcCmdNoWindow,
+  SvcCmdHelp,
   NumSvcCmds
 };
 
@@ -100,7 +101,8 @@ static const char * const ServiceCommandNames[NumSvcCmds] = {
   "Deinstall",
   "AutoRestart",
   "NoRestart",
-  "NoWin"
+  "NoWin",
+  "Help"
 };
 
 
@@ -209,7 +211,7 @@ class PSystemLogToEvent : public PSystemLogTarget
     if (thread != NULL)
       threadName = thread->GetThreadName();
     else
-      threadName.sprintf("%u", GetCurrentThreadId());
+      threadName.sprintf(PTHREAD_ID_FMT, GetCurrentThreadId());
 
     char thrdbuf[16];
     if (threadName.IsEmpty())
@@ -334,8 +336,10 @@ int PServiceProcess::InternalMain(void * arg)
   SetLogLevel(debugMode ? PSystemLog::Info : PSystemLog::Warning);
 
   if (!debugMode && arguments.GetCount() > 0) {
-    for (PINDEX a = 0; a < arguments.GetCount(); a++)
-      ProcessCommand(arguments[a]);
+    for (PINDEX a = 0; a < arguments.GetCount(); a++) {
+      if (!ProcessCommand(arguments[a]))
+        return 1;
+    }
 
     if (controlWindow == NULL || controlWindow == (HWND)-1)
       return GetTerminationValue();
@@ -440,10 +444,10 @@ int PServiceProcess::InternalMain(void * arg)
 
   // Set thread ID for process to this thread
   m_activeThreadMutex.Wait();
-  m_activeThreads.erase(threadId);
-  threadId = GetCurrentThreadId();
+  m_activeThreads.erase(m_threadId);
+  m_threadId = GetCurrentThreadId();
   threadHandle = GetCurrentThread();
-  m_activeThreads[threadId] = this;
+  m_activeThreads[m_threadId] = this;
   m_activeThreadMutex.Signal();
   OnStop();
 
@@ -1019,9 +1023,9 @@ void PServiceProcess::StaticThreadEntry(void * arg)
 void PServiceProcess::ThreadEntry()
 {
   m_activeThreadMutex.Wait();
-  threadId = GetCurrentThreadId();
+  m_threadId = ::GetCurrentThreadId();
   threadHandle = GetCurrentThread();
-  m_activeThreads[threadId] = this;
+  m_activeThreads[m_threadId] = this;
   m_activeThreadMutex.Signal();
 
   SetTerminationValue(1);
@@ -1519,21 +1523,15 @@ bool NT_ServiceManager::IsRunning(PServiceProcess * svc)
 
 bool PServiceProcess::ProcessCommand(const char * cmd)
 {
+  if (*cmd == '/')
+    ++cmd;
+  else if (cmd[0] == '-' && cmd[1] == '-')
+    cmd += 2;
+
   PINDEX cmdNum = 0;
   while (strcasecmp(cmd, ServiceCommandNames[cmdNum]) != 0) {
-    if (++cmdNum >= NumSvcCmds) {
-      if (!CreateControlWindow(true))
-        return false;
-      if (*cmd != '\0')
-        PError << "Unknown command \"" << cmd << "\".\n";
-      else
-        PError << "Could not start service.\n";
-      PError << "usage: " << GetName() << " [ ";
-      for (cmdNum = 0; cmdNum < NumSvcCmds-1; cmdNum++)
-        PError << ServiceCommandNames[cmdNum] << " | ";
-      PError << ServiceCommandNames[cmdNum] << " ]" << endl;
-      return false;
-    }
+    if (++cmdNum >= NumSvcCmds)
+      break;
   }
 
   NT_ServiceManager nt;
@@ -1545,12 +1543,14 @@ bool PServiceProcess::ProcessCommand(const char * cmd)
   else
     svcManager = &nt;
 
+  PStringStream msg;
   bool good = false;
+
   switch (cmdNum) {
     case SvcCmdNoWindow :
       if (controlWindow == NULL)
         controlWindow = (HWND)-1;
-      break;
+      return true;
 
     case SvcCmdTray :
       if (CreateControlWindow(false)) {
@@ -1576,17 +1576,11 @@ bool PServiceProcess::ProcessCommand(const char * cmd)
       return true;
 
     case SvcCmdVersion : // Version command
-      ::SetLastError(0);
-      {
-        PStringStream msg;
-        msg << GetName() << " Version " << GetVersion(true)
-            << " by " << GetManufacturer()
-            << " on " << GetOSClass()   << ' ' + GetOSName()
-            << " ("   << GetOSVersion() << '-' + GetOSHardware() << ')';
-        MessageBox(NULL, msg, GetName(), MB_TASKMODAL);
-        PError << msg << endl;
-      }
-      return true;
+      msg << GetName() << " Version " << GetVersion(true)
+          << " by " << GetManufacturer()
+          << " on " << GetOSClass()   << ' ' + GetOSName()
+          << " ("   << GetOSVersion() << '-' + GetOSHardware() << ')';
+      break;
 
     case SvcCmdDefault : // run app with no params.
       if (svcManager->IsInstalled(this)) {
@@ -1670,39 +1664,50 @@ bool PServiceProcess::ProcessCommand(const char * cmd)
       svcManager->Remove(this);
       TrayIconRegistry(this, DelTrayIcon);
 #if P_CONFIG_FILE
-      PConfig cfg;
-      PStringArray sections = cfg.GetSections();
-      PINDEX i;
-      for (i = 0; i < sections.GetSize(); i++)
-        cfg.DeleteSection(sections[i]);
+      {
+        PConfig cfg;
+        PStringArray sections = cfg.GetSections();
+        PINDEX i;
+        for (i = 0; i < sections.GetSize(); i++)
+          cfg.DeleteSection(sections[i]);
+      }
 #endif // P_CONFIG_FILE
       good = true;
       break;
+
+    default :
+      if (cmdNum != SvcCmdHelp)
+        msg << "Unknown command \"" << cmd << "\".\r\n\r\n";
+      msg << "usage: " << GetFile().GetTitle() << " [ ";
+      for (cmdNum = 0; cmdNum < NumSvcCmds-1; cmdNum++)
+        msg << ServiceCommandNames[cmdNum] << " | ";
+      msg << ServiceCommandNames[cmdNum] << " ]" << endl;
   }
 
   SetLastError(0);
 
-  PStringStream msg;
-  msg << "Service command ";
-  if (cmdNum != SvcCmdDefault)
-    msg << '"' << ServiceCommandNames[cmdNum] << "\" ";
-  if (good)
-    msg << "successful.";
-  else {
-    msg << "failed - ";
-    switch (svcManager->GetError()) {
-      case ERROR_ACCESS_DENIED :
-        msg << "Access denied";
-        break;
-      case 0x10000000 :
-        msg << "process still running.";
-        break;
-      default :
-        msg << "error code = " << svcManager->GetError();
+  if (msg.IsEmpty()) {
+    msg << "Service command ";
+    if (cmdNum != SvcCmdDefault)
+      msg << '"' << ServiceCommandNames[cmdNum] << "\" ";
+    if (good)
+      msg << "successful.";
+    else {
+      msg << "failed - ";
+      switch (svcManager->GetError()) {
+        case ERROR_ACCESS_DENIED :
+          msg << "Access denied";
+          break;
+        case 0x10000000 :
+          msg << "process still running.";
+          break;
+        default :
+          msg << "error code = " << svcManager->GetError();
+      }
     }
   }
 
-  if (controlWindow == NULL || controlWindow == (HWND)-1)
+  if (controlWindow == NULL)
     MessageBox(NULL, msg, GetName(), MB_TASKMODAL);
   else
     PError << msg << endl;
