@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26575 $
+ * $Revision: 28051 $
  * $Author: rjongbloed $
- * $Date: 2011-10-10 06:11:12 -0500 (Mon, 10 Oct 2011) $
+ * $Date: 2012-07-18 02:20:34 -0500 (Wed, 18 Jul 2012) $
  */
 
 
@@ -63,8 +63,12 @@ static PBoolean PAssertAction(int c, const char * msg)
     case 'd' :
     case 'D' :
       {
-        PString cmd = "gdb " + PProcess::Current().GetFile();
+        PString cmd = ::getenv("PTLIB_ASSERT_DEBUGGER");
+        if (cmd.IsEmpty())
+          cmd = "gdb";
+        cmd &= PProcess::Current().GetFile();
         cmd.sprintf(" %d", getpid());
+        PError << "\nStarting debugger \"" << cmd << "\"\n";
         system((const char *)cmd);
       }
       break;
@@ -116,14 +120,18 @@ void PAssertFunc(const char * msg)
 
 #if P_EXCEPTIONS
   //Throw a runtime exception if the environment variable PWLIB_ASSERT_EXCEPTION is set
-  env = ::getenv("PWLIB_ASSERT_EXCEPTION");
+  env = ::getenv("PTLIB_ASSERT_EXCEPTION");
+  if (env == NULL)
+    env = ::getenv("PWLIB_ASSERT_EXCEPTION");
   if (env != NULL){
     throw std::runtime_error(msg);
   }
 #endif
   
 #ifndef P_VXWORKS
-  env = ::getenv("PWLIB_ASSERT_ACTION");
+  env = ::getenv("PTLIB_ASSERT_ACTION");
+  if (env == NULL)
+    env = ::getenv("PWLIB_ASSERT_ACTION");
   if (env != NULL && *env != EOF && PAssertAction(*env, msg)) {
     inAssert = PFalse;
     return;
