@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26711 $
+ * $Revision: 28050 $
  * $Author: rjongbloed $
- * $Date: 2011-11-29 23:56:17 -0600 (Tue, 29 Nov 2011) $
+ * $Date: 2012-07-18 02:15:30 -0500 (Wed, 18 Jul 2012) $
  */
 
 #pragma implementation "channel.h"
@@ -106,7 +106,8 @@ PBoolean PChannel::PXSetIOBlock(PXBlockType type, const PTimeInterval & timeout)
       case PXReadBlock :
         if (px_readThread != NULL && px_lastBlockType == PXReadBlock)
           PAssertAlways(psprintf("Attempt to do simultaneous reads from multiple threads:"
-                                 " os_handle=%i, thread ID=%X", os_handle, px_readThread->GetThreadId()));
+                                 " os_handle=%i, thread ID=" PTHREAD_ID_FMT,
+                                 os_handle, px_readThread->GetThreadId()));
         // Fall into default case
 
       default :
