@@ -26,9 +26,9 @@
  *
  * Contributor(s): Yuri Kiryanov, ykiryanov at users.sourceforge.net
  *
- * $Revision: 25645 $
+ * $Revision: 27623 $
  * $Author: rjongbloed $
- * $Date: 2011-05-01 19:19:49 -0500 (Sun, 01 May 2011) $
+ * $Date: 2012-05-14 19:03:37 -0500 (Mon, 14 May 2012) $
  */
 
 class PThread;
@@ -459,6 +459,9 @@ void PHouseKeepingThread::Main()
 
 void PProcess::SignalTimerChange()
 {
+  if (!PAssert(IsInitialised(), PLogicError) || m_shuttingDown) 
+    return false;
+
   if (housekeepingThread == NULL)
   {  
     housekeepingThread = new PHouseKeepingThread;
