@@ -26,9 +26,9 @@
  *
  * Contributor(s):  ______________________________________.
  *
- * $Revision: 21219 $
+ * $Revision: 27623 $
  * $Author: rjongbloed $
- * $Date: 2008-10-02 03:52:00 -0500 (Thu, 02 Oct 2008) $
+ * $Date: 2012-05-14 19:03:37 -0500 (Mon, 14 May 2012) $
  */
 
 
@@ -501,6 +501,9 @@ void PProcess::HouseKeepingThread::Main()
 
 void PProcess::SignalTimerChange()
 {
+  if (!PAssert(IsInitialised(), PLogicError) || m_shuttingDown) 
+    return false;
+
   if (houseKeeper == NULL) {
     // Prevent reentrance before the following assignment is done
     // Placed after above if-statement due to efficiency, and so 
